package com.intellij.hub.auth;

/*Generated by MPS */

import org.jetbrains.annotations.NotNull;
import com.intellij.hub.auth.oauth2.error.OAuthAuthErrorCode;
import org.jetbrains.annotations.Nullable;
import java.util.Arrays;

public class OAuthClientValidationException extends OAuthValidationException {
  private static final String DOC_PREFIX = "auth";

  /**
   * All exception thrown after client checks shoud be OAuthClientProblemException
   * 
   * @param code oauth error code
   */
  private OAuthClientValidationException(@NotNull final OAuthAuthErrorCode code) {
    this(code, null);
  }

  /**
   * All exception thrown after client checks shoud be OAuthClientProblemException
   * 
   * @param code oauth error code
   * @param message optional message
   */
  private OAuthClientValidationException(@NotNull final OAuthAuthErrorCode code, @Nullable String message) {
    super(code, message, DOC_PREFIX);
  }

  public static OAuthValidationException missingParameter(String parameterName) {
    // Used in both auth and token scenarious 
    return new OAuthClientValidationException(OAuthAuthErrorCode.INVALID_REQUEST, "Parameter '" + parameterName + "' is required");
  }

  public static OAuthValidationException accessDenied(String reason) {
    return new OAuthClientValidationException(OAuthAuthErrorCode.ACCESS_DENIED, reason);
  }

  public static OAuthClientValidationException invalidScope(String message) {
    // Used in both auth and token scenarious 
    return new OAuthClientValidationException(OAuthAuthErrorCode.INVALID_SCOPE, message);
  }

  public static OAuthClientValidationException unknownScope(String[] scope) {
    // Used in both but should be handled differnectly depending on exception type 
    return invalidScope("Scope " + Arrays.toString(scope) + " doesn't match any Service");
  }

  public static OAuthClientValidationException invalidCSRFCode() {
    return new OAuthClientValidationException(OAuthAuthErrorCode.SERVER_ERROR, "Invalid CSRF token recieved.");
  }

  public static OAuthValidationException cantGiveGuestOfflineAccess() {
    return new OAuthClientValidationException(OAuthAuthErrorCode.INVALID_REQUEST, "Unable request offline access for Guest");
  }

  public static OAuthValidationException invalidResponseType(String responseType) {
    return new OAuthClientValidationException(OAuthAuthErrorCode.UNSUPPORTED_RESPONSE_TYPE, "Response type '" + responseType + "' is unknown or unsupported");
  }

  public static OAuthValidationException invalidParameter(String parameterName) {
    return new OAuthClientValidationException(OAuthAuthErrorCode.INVALID_REQUEST, "Parameter '" + parameterName + "' is invalid");
  }

  public static OAuthClientValidationException unknownService(String serviceId) {
    return new OAuthClientValidationException(OAuthAuthErrorCode.INVALID_REQUEST, "Service with id \"" + serviceId + "\" was not found");
  }

  public static OAuthValidationException serviceHasNoRedirectURI(String clientServiceName, String requestedRedirectURI) {
    return new OAuthClientValidationException(OAuthAuthErrorCode.UNAUTHORIZED_CLIENT, "Unregistered Redirect URI. Client service '" + clientServiceName + "' is not authorized to request an authorization entity using this method because the service has no registered " + "'Redirect URIs'. If you have Hub administrator permissions and the URI " + requestedRedirectURI + " does belong to '" + clientServiceName + "', please add this URI to the list of 'Redirect URIs' in the service settings in Hub.");
  }

  public static OAuthValidationException unregisteredRedirectURI(String clientServiceName, String requestedRedirectURI) {
    return new OAuthClientValidationException(OAuthAuthErrorCode.UNAUTHORIZED_CLIENT, "Unregistered Redirect URI. Client service '" + clientServiceName + "' requests authorization entity for the URI which is not registered in Hub: " + requestedRedirectURI + ". If you have Hub administrator permissions and the suspicious URI does belong to '" + clientServiceName + "', please add this URI to the list of 'Redirect URIs' in the service settings in Hub");
  }
}
