package com.intellij.hub.auth;

/*Generated by MPS */

import org.jetbrains.annotations.NotNull;
import com.intellij.hub.auth.oauth2.error.OAuthTokenErrorCode;
import org.jetbrains.annotations.Nullable;
import java.util.Arrays;

public class OAuthTokenValidationException extends OAuthValidationException {
  private static final String DOC_PREFIX = "token";

  private OAuthTokenValidationException(@NotNull final OAuthTokenErrorCode code) {
    this(code, null);
  }

  private OAuthTokenValidationException(@NotNull final OAuthTokenErrorCode code, @Nullable String description) {
    super(code, description, DOC_PREFIX);
  }

  public static OAuthValidationException missingParameter(String parameterName) {
    // Used in both auth and token scenarious 
    return new OAuthTokenValidationException(OAuthTokenErrorCode.INVALID_REQUEST, "Parameter '" + parameterName + "' is required");
  }

  public static OAuthTokenValidationException invalidScope(String message) {
    // Used in both auth and token scenarious 
    return new OAuthTokenValidationException(OAuthTokenErrorCode.INVALID_SCOPE, message);
  }

  public static OAuthTokenValidationException unknownScope(String[] scope) {
    // Used in both but should be handled differnectly depending on exception type 
    return invalidScope("Scope " + Arrays.toString(scope) + " doesn't match any Service");
  }

  public static OAuthValidationException missingAuthHeader() {
    return new OAuthTokenValidationException(OAuthTokenErrorCode.INVALID_CLIENT, "Authorization header was not passed");
  }

  public static OAuthValidationException invalidGrant() {
    return new OAuthTokenValidationException(OAuthTokenErrorCode.UNSUPPORTED_GRANT_TYPE);
  }

  public static OAuthValidationException invalidServiceSecret() {
    return new OAuthTokenValidationException(OAuthTokenErrorCode.INVALID_GRANT, "Invalid service secret");
  }

  public static OAuthValidationException unknownAuthCode() {
    return new OAuthTokenValidationException(OAuthTokenErrorCode.INVALID_GRANT, "Unknown authorization code");
  }

  public static OAuthValidationException invalidAuthCode() {
    return new OAuthTokenValidationException(OAuthTokenErrorCode.INVALID_GRANT, "Invalid authorization code");
  }

  public static OAuthValidationException wrongClient() {
    return new OAuthTokenValidationException(OAuthTokenErrorCode.INVALID_GRANT, "Refresh token is issued to another client");
  }

  public static OAuthValidationException unknownRefreshToken() {
    return new OAuthTokenValidationException(OAuthTokenErrorCode.INVALID_GRANT, "Unknown refresh token");
  }

  public static OAuthValidationException inaccessibleScope() {
    return new OAuthTokenValidationException(OAuthTokenErrorCode.INVALID_GRANT, "Requested scope does not match allowed by access token");
  }

  public static OAuthValidationException invalidCredentials() {
    return new OAuthTokenValidationException(OAuthTokenErrorCode.INVALID_GRANT, "Invalid resource owner credentials");
  }

  public static OAuthValidationException untrustedService() {
    return new OAuthTokenValidationException(OAuthTokenErrorCode.UNAUTHORIZED_CLIENT, "Service is not verified");
  }

  public static OAuthValidationException invalidAuthHeader(Throwable cause) {
    OAuthTokenValidationException e = new OAuthTokenValidationException(OAuthTokenErrorCode.INVALID_CLIENT, "Invalid authorization header");
    e.initCause(cause);
    return e;
  }

  public static OAuthValidationException unknownService(String clientServiceId) {
    // Used in both auth and token scenarious 
    return new OAuthTokenValidationException(OAuthTokenErrorCode.INVALID_CLIENT, "Service with id \"" + clientServiceId + "\" was not found");
  }
}
