package com.intellij.hub.auth.basic;

/*Generated by MPS */

import com.intellij.hub.auth.AuthToken;
import java.nio.charset.Charset;
import org.jetbrains.annotations.NotNull;
import javax.xml.bind.DatatypeConverter;
import com.intellij.hub.auth.InvalidTokenException;
import java.util.regex.Pattern;
import com.intellij.hub.auth.InvalidTokenDataException;
import com.intellij.hub.auth.InvalidAuthorizationHeaderException;


/**
 * * @author Danila Ponomarenko
 */
public final class BasicAuthToken implements AuthToken {
  private static final Charset DEFAULT_CHARSET = Charset.forName("utf-8");
  public static final String BASIC_TYPE = "Basic";
  public static final String SEMICOLON = ":";
  public static final String BASIC_PREFIX = BASIC_TYPE + " ";

  private final String username;
  private final String password;

  public BasicAuthToken(@NotNull String username, @NotNull String password) {
    this.username = username;
    this.password = password;
  }

  @Override
  @NotNull
  public String encode() {
    return DatatypeConverter.printBase64Binary((username + SEMICOLON + password).getBytes(DEFAULT_CHARSET));
  }

  @NotNull
  public String getUsername() {
    return username;
  }

  @NotNull
  public String getPassword() {
    return password;
  }

  @NotNull
  @Override
  public String getType() {
    return BASIC_TYPE;
  }

  @Override
  public boolean equals(Object o) {
    if (this == o) {
      return true;
    }
    if (o == null || getClass() != o.getClass()) {
      return false;
    }
    BasicAuthToken that = (BasicAuthToken) o;
    return password.equals(that.password) && username.equals(that.username);
  }

  @Override
  public int hashCode() {
    int result = username.hashCode();
    result = 31 * result + password.hashCode();
    return result;
  }

  @NotNull
  public static BasicAuthToken decode(@NotNull String token) throws InvalidTokenException {
    final String decodedB64Token = new String(DatatypeConverter.parseBase64Binary(token), DEFAULT_CHARSET);
    final String[] parts = decodedB64Token.split(Pattern.quote(SEMICOLON));
    if (parts.length == 1) {
      return new BasicAuthToken(parts[0], "");
    }
    if (parts.length == 2) {
      return new BasicAuthToken(parts[0], parts[1]);
    }
    throw new InvalidTokenDataException("Unable to decode basic token. Decoded parts length: " + parts.length);
  }

  @NotNull
  public static BasicAuthToken decodeHeader(@NotNull String header) throws InvalidTokenException {
    if (!(isBasic(header))) {
      throw new InvalidAuthorizationHeaderException(BASIC_TYPE);
    }
    return BasicAuthToken.decode(header.substring(BASIC_PREFIX.length()));
  }

  @NotNull
  public static String encodeHeader(@NotNull BasicAuthToken token) {
    return BASIC_PREFIX + token.encode();
  }

  public static boolean isBasic(String header) {
    return (header != null && header.length() > 0) && header.startsWith(BASIC_PREFIX);
  }
}
