package com.intellij.hub.auth.oauth2.provider.verifier;

/*Generated by MPS */

import com.intellij.hub.auth.oauth2.provider.source.DSAPublicKeySource;
import org.jetbrains.annotations.NotNull;
import com.intellij.hub.auth.oauth2.token.AccessToken;
import com.intellij.hub.auth.InvalidTokenSignatureException;
import java.security.Signature;


/**
 * * @author Danila Ponomarenko
 */
public class SHA1withDSAVerifier implements TokenVerifier {
  public static final String PROVIDER = "SUN";
  public static final String SIGNATURE_ALGORITHM = "SHA1with" + DSAPublicKeySource.DSA_KEY_ALGORITHM;

  private final DSAPublicKeySource publicKeySource;

  public SHA1withDSAVerifier(@NotNull DSAPublicKeySource publicKey) {
    this.publicKeySource = publicKey;
  }

  @Override
  public void verify(@NotNull AccessToken token) throws InvalidTokenSignatureException {
    if (token.getKeyVersion() > publicKeySource.getVersion()) {
      publicKeySource.forceUpdate();
    }
    final Signature sig;
    try {
      sig = Signature.getInstance(SIGNATURE_ALGORITHM, PROVIDER);
      sig.initVerify(publicKeySource.getKey());
      sig.update(token.getData().getBytes());
      if (!(sig.verify(token.getSignature()))) {
        throw new InvalidTokenSignatureException();
      }
    } catch (Exception e) {
      throw new InvalidTokenSignatureException(e);
    }
  }
}
