package com.intellij.hub.auth.oauth2.token;

/*Generated by MPS */

import java.nio.charset.Charset;
import java.util.Date;
import org.jetbrains.annotations.NotNull;
import org.jetbrains.annotations.Nullable;
import java.util.Arrays;
import java.util.regex.Pattern;
import com.intellij.hub.auth.InvalidTokenDataException;


/**
 * * @author Danila Ponomarenko
 */
public class AccessTokenData {
  public static final Charset DEFAULT_CHARSET = Charset.forName("utf-8");
  public static final String DELIMITER = ".";

  private final Date expiration;
  private final String client;
  private final String user;
  private final String[] scope;
  private final int ttlSeconds;

  private AccessTokenData(@NotNull Date expiration, int timeToLiveSeconds, @NotNull String client, @Nullable String user, @NotNull String... scope) {
    this.expiration = expiration;
    this.ttlSeconds = timeToLiveSeconds;
    this.client = client;
    this.user = user;
    this.scope = scope;
  }

  public AccessTokenData(@NotNull Date expiration, @NotNull String client, @Nullable String user, @NotNull String... target) {
    this(expiration, -1, client, user, target);
  }

  public AccessTokenData(@NotNull int timeToLiveSeconds, @NotNull String client, @Nullable String user, @NotNull String... target) {
    this(new Date(System.currentTimeMillis() + timeToLiveSeconds * 1000), timeToLiveSeconds, client, user, target);
  }

  /**
   * Returns access token expiration period in seconds
   * 
   * @return period when token is valid. Negative values - unknown
   */
  public int getTTLSeconds() {
    return ttlSeconds;
  }

  @NotNull
  public Date getExpiration() {
    return expiration;
  }

  @NotNull
  public String getClient() {
    return client;
  }

  @Nullable
  public String getUser() {
    return user;
  }

  @NotNull
  public String[] getScope() {
    return scope;
  }

  @NotNull
  public byte[] getBytes() {
    return encode().getBytes(DEFAULT_CHARSET);
  }

  @NotNull
  public String encode() {
    return expiration.getTime() + DELIMITER + client + DELIMITER + ((user != null ?
      user :
      ""
    )) + DELIMITER + ScopeUtil.encodeScope(scope);
  }

  @Override
  public boolean equals(Object o) {
    if (this == o) {
      return true;
    }
    if (o == null || getClass() != o.getClass()) {
      return false;
    }
    AccessTokenData tokenData = (AccessTokenData) o;
    return client.equals(tokenData.client) && Arrays.equals(scope, tokenData.scope) && expiration.equals(tokenData.expiration);
  }

  @Override
  public int hashCode() {
    int result = expiration.hashCode();
    result = 31 * result + client.hashCode();
    result = 31 * result + Arrays.hashCode(scope);
    return result;
  }

  @Override
  public String toString() {
    return String.format("%s{expiration=%s, client=%s, user=%s, scope=%s}", this.getClass().getSimpleName(), expiration, client, user, Arrays.toString(scope));
  }

  @NotNull
  public static AccessTokenData decode(@NotNull String data) {
    final String[] parts = data.split(Pattern.quote(DELIMITER));
    if (parts.length != 4) {
      throw new InvalidTokenDataException("Invalid token data");
    }
    return new AccessTokenData(new Date(Long.valueOf(parts[0])), parts[1], (parts[2].isEmpty() ?
      null :
      parts[2]
    ), ScopeUtil.decodeScope(parts[3]));
  }
}
