package com.intellij.hub.core;

/*Generated by MPS */

import org.jetbrains.annotations.NotNull;
import org.jetbrains.annotations.Nullable;
import java.net.URI;

public interface Error {
  public static final String HUB_DOCUMENTATION_SPACE = "https://www.jetbrains.com/hub/help/1.0";
  public static final String OAUTH2_ERROR_PAGE = Error.HUB_DOCUMENTATION_SPACE + "/OAuth-2.0-Errors.html";
  public static final String HUB_REST_API = Error.HUB_DOCUMENTATION_SPACE + "/HUB-REST-API";

  /**
   * Error phrase of error or ASCII error code
   * Like Conflict, token_expired, etc...
   * 
   * @return code or status phrase
   */
  @NotNull
  public String getError();
  /**
   * Error specific status code (404, 401, 500)
   * It can be extended code like 4097 (conflict in entity table 7 or something for specific problem)
   * 
   * @return error code
   */
  public int getStatusCode();
  /**
   * User friendly error description.
   * 
   * @return error description
   */
  @NotNull
  public String getMessage();
  /**
   * Low level devolper message
   * 
   * @return error may return developer message for debug purposes
   */
  @Nullable
  public String getDeveloperMessage();
  /**
   * Get error description page. Optional
   * 
   * @return null - if not help link provided
   */
  @Nullable
  public URI getURI();
  /**
   * Field name that the error is associated with
   * 
   * @return null - if the error is not associated with a field
   */
  @Nullable
  public String getField();
}
