package com.intellij.hub.core.data.uri;

/*Generated by MPS */

import java.util.regex.Pattern;
import javax.xml.bind.DatatypeConverter;
import java.io.InputStream;
import java.io.IOException;
import java.util.regex.Matcher;
import java.io.ByteArrayOutputStream;


/**
 * * Converts data uri back and forward
 * *
 * * @author Maxim Mazin
 */
public class DataURI {
  private static final Pattern DATA_URI_PATTERN = Pattern.compile("data\\:(.+?);base64,(.+)", Pattern.CASE_INSENSITIVE | Pattern.DOTALL);

  private String mimeType;
  private String base64Content;

  public DataURI(String mimeType, byte[] content) {
    this.mimeType = mimeType;
    this.base64Content = DatatypeConverter.printBase64Binary(content);
  }

  public DataURI(String mimeType, InputStream content) throws IOException {
    this(mimeType, toByteArray(content));
  }

  public DataURI(String dataURI) {
    Matcher matcher = DATA_URI_PATTERN.matcher(dataURI);
    if (matcher.matches()) {
      mimeType = matcher.group(1);
      base64Content = matcher.group(2);
    } else {
      throw new IllegalArgumentException("Bad Data URI format: " + dataURI);
    }
  }

  public String getDataURI() {
    return "data:" + mimeType + ";base64," + base64Content;
  }

  public String getMimeType() {
    return mimeType;
  }

  public byte[] getContent() {
    return DatatypeConverter.parseBase64Binary(base64Content);
  }

  public String getBase64Content() {
    return base64Content;
  }

  protected static byte[] toByteArray(InputStream content) throws IOException {
    ByteArrayOutputStream out = new ByteArrayOutputStream();
    byte[] buffer = new byte[1024];
    int l;
    while ((l = content.read(buffer)) >= 0) {
      out.write(buffer, 0, l);
    }
    out.close();
    content.close();
    return out.toByteArray();
  }
}
