package jetbrains.jetpass.client;

/*Generated by MPS */

import java.util.Set;
import java.util.LinkedHashSet;
import java.util.Map;
import java.util.LinkedHashMap;
import javax.ws.rs.client.WebTarget;

public class FieldPartial<F extends BaseField> {
  private static final String FIELDS_KEYWORD = "fields";

  private final Set<F> fields;

  public FieldPartial(F... fields) {
    this(convertArrayToSet(fields));
  }

  public FieldPartial(Iterable<F> fields) {
    this(convertIterableToSet(fields));
  }

  public FieldPartial(Set<F> fields) {
    if (fields.isEmpty()) {
      throw new IllegalArgumentException("Fields can't be empty");
    }
    this.fields = fields;
  }

  public Set<F> getFields() {
    return fields;
  }

  public Set<String> getFieldNameSet() {
    Set<String> set = new LinkedHashSet<String>(this.fields.size());
    for (F field : this.fields) {
      set.add(field.getName());
    }
    return set;
  }

  public FieldPartial<F> union(FieldPartial<F> that) {
    return new FieldPartial<F>(this.unionFields(that));
  }

  protected Set<F> unionFields(FieldPartial<F> that) {
    Map<String, F> thatFields = new LinkedHashMap<String, F>();
    for (F thatField : that.fields) {
      thatFields.put(thatField.getName(), thatField);
    }

    Set<F> newFields = new LinkedHashSet<F>();
    for (F thisField : this.fields) {
      F thatField = thatFields.remove(thisField.getName());
      if (thatField != null) {
        newFields.add((F) thisField.union(thatField));
      } else {
        newFields.add(thisField);
      }
    }

    for (F thatField : thatFields.values()) {
      newFields.add(thatField);
    }

    return newFields;
  }

  @Override
  public String toString() {
    StringBuilder builder = new StringBuilder();
    boolean first = true;
    for (F field : fields) {
      if (!(first)) {
        builder.append(',');
      }
      first = false;
      field.append(builder);

    }
    return builder.toString();
  }

  private static <T> Set<T> convertArrayToSet(T[] arr) {
    Set<T> set = new LinkedHashSet<T>(arr.length);
    for (int i = 0; i < arr.length; i++) {
      set.add(arr[i]);
    }
    return set;
  }

  private static <T> Set<T> convertIterableToSet(Iterable<T> iterable) {
    Set<T> set = new LinkedHashSet<T>();
    for (T field : iterable) {
      set.add(field);
    }
    return set;
  }

  public static WebTarget apply(WebTarget resource, FieldPartial partial) {
    if (partial != null) {
      resource = resource.queryParam(FIELDS_KEYWORD, BaseClient.wrapObject(partial));
    }
    return resource;
  }
}
