package jetbrains.jetpass.client.accounts;

/*Generated by MPS */

import com.intellij.hub.auth.oauth2.consumer.TokenHolder;
import org.jetbrains.annotations.Nullable;
import jetbrains.jetpass.rest.dto.ServiceJSON;
import org.jetbrains.annotations.NotNull;
import javax.ws.rs.client.WebTarget;
import jetbrains.jetpass.client.resource.RequestValidator;
import jetbrains.jetpass.client.hub.HubClient;
import java.net.URL;
import java.net.MalformedURLException;
import java.util.UUID;
import javax.ws.rs.WebApplicationException;
import java.net.UnknownHostException;
import java.net.ConnectException;
import java.net.SocketTimeoutException;
import jetbrains.jetpass.rest.dto.ErrorJSON;
import com.intellij.hub.auth.oauth2.error.OAuthTokenErrorCode;
import javax.ws.rs.ProcessingException;
import javax.ws.rs.core.Response;
import com.intellij.hub.auth.oauth2.consumer.OAuth2TokenLoadException;
import javax.ws.rs.client.Client;
import org.glassfish.jersey.client.ClientProperties;
import jetbrains.jetpass.rest.dto.UserJSON;
import jetbrains.jetpass.rest.dto.SpaceJSON;
import jetbrains.jetpass.rest.dto.PermissionJSON;
import jetbrains.jetpass.client.hub.HubURLBuilder;

public abstract class BaseAccountsClient {
  private final TokenHolder holder;

  protected BaseAccountsClient(@Nullable final TokenHolder holder) {
    this.holder = holder;
  }

  @Nullable
  public String getClientServiceId() {
    ServiceJSON myService = getCurrentService();
    return (myService != null ?
      myService.getId() :
      null
    );
  }

  @Nullable
  public TokenHolder getTokenHolder() {
    return holder;
  }

  @Nullable
  public abstract String getBaseURL();

  @NotNull
  public abstract WebTarget getRootResource();

  @NotNull
  public abstract RequestValidator getOAuthRequestValidator();

  @NotNull
  public abstract HubClient getHubClient();

  @NotNull
  public ServiceCredentialsValidationResult checkServiceCredentials() {
    String baseURL = getBaseURL();
    if ((baseURL == null || baseURL.length() == 0)) {
      return new ServiceCredentialsValidationResult(ServiceCredentialsValidationResult.Status.EMPTY_URL);
    }
    try {
      new URL(baseURL);
    } catch (MalformedURLException e) {
      return new ServiceCredentialsValidationResult(ServiceCredentialsValidationResult.Status.MALFORMED_URL, ServiceCredentialsValidationResult.Status.MALFORMED_URL.getMessage(), e);
    }

    try {
      // Try to get hub scope 
      String scope = getHubClient().getOAuthClient().getHubScope();
      try {
        UUID.fromString(scope);
      } catch (RuntimeException ex) {
        // if we can't parse UUID than server returns somtehting wrong 
        return new ServiceCredentialsValidationResult(ServiceCredentialsValidationResult.Status.WRONG_URL, "Unknown target response: " + scope, ex);
      }
    } catch (WebApplicationException e) {
      return new ServiceCredentialsValidationResult(ServiceCredentialsValidationResult.Status.WRONG_URL, e.getMessage(), e);
    } catch (RuntimeException e) {
      Throwable t = e;
      while (t instanceof RuntimeException && t.getCause() != null && t.getCause() != t) {
        t = t.getCause();
      }
      ServiceCredentialsValidationResult.Status status;
      if (t instanceof UnknownHostException) {
        status = ServiceCredentialsValidationResult.Status.UNKNOWN_HOST;
      } else if (t instanceof ConnectException) {
        status = ServiceCredentialsValidationResult.Status.CANT_CONNECT;
      } else if (t instanceof SocketTimeoutException) {
        status = ServiceCredentialsValidationResult.Status.TIMEOUT;
      } else {
        status = ServiceCredentialsValidationResult.Status.CANT_CONNECT;
      }
      return new ServiceCredentialsValidationResult(status, t.getMessage(), t);
    }

    TokenHolder holder = this.getTokenHolder();
    if (holder == null) {
      return new ServiceCredentialsValidationResult(ServiceCredentialsValidationResult.Status.SERVICE_CREDENTIALS_UNDEFINED);
    }


    try {
      // Try to get auth token and header 
      holder.getHeader();

      try {
        // Try to access service 
        ServiceJSON theService = getCurrentService();
        if (theService == null) {
          return new ServiceCredentialsValidationResult(ServiceCredentialsValidationResult.Status.SERVICE_NOT_REGISTERED, "Service was not found in Hub");
        } else if (!(theService.isTrusted())) {
          return new ServiceCredentialsValidationResult(ServiceCredentialsValidationResult.Status.SERVICE_NOT_VERIFIED, ServiceCredentialsValidationResult.Status.SERVICE_NOT_VERIFIED.getMessage());
        } else {
          return new ServiceCredentialsValidationResult(ServiceCredentialsValidationResult.Status.OK);
        }
      } catch (WebApplicationException ex) {
        try {
          ErrorJSON error = ex.getResponse().readEntity(ErrorJSON.class);
          if (error != null) {
            if (eq_sjvc5t_a0a0b0a0a3a01a6(error.getError(), OAuthTokenErrorCode.INVALID_CLIENT.getASCIICode())) {
              // Service seems to be removed in HUB after AccessToken grant 
              return new ServiceCredentialsValidationResult(ServiceCredentialsValidationResult.Status.SERVICE_NOT_REGISTERED, ServiceCredentialsValidationResult.Status.SERVICE_NOT_REGISTERED.getMessage(), ex);
            } else if (eq_sjvc5t_a0a0a1a0a0d0k0g(error.getError(), OAuthTokenErrorCode.UNAUTHORIZED_CLIENT.getASCIICode())) {
              // Service seems to be unverified in HUB after AccessToken grant 
              // JPS-1789 should not happen anymore or lost it's meaning 
              return new ServiceCredentialsValidationResult(ServiceCredentialsValidationResult.Status.SERVICE_NOT_VERIFIED, ServiceCredentialsValidationResult.Status.SERVICE_NOT_VERIFIED.getMessage(), ex);
            } else {
              // Something other happened 
              return new ServiceCredentialsValidationResult(ServiceCredentialsValidationResult.Status.OTHER, (error.getError_description() == null ?
                "Error is " + error.getError() :
                error.getError_description()
              ), ex);
            }
          }
        } catch (ProcessingException e) {
          // Can't get response entity 
        }
        if (ex.getResponse().getStatusInfo() == Response.Status.FORBIDDEN) {
          return new ServiceCredentialsValidationResult(ServiceCredentialsValidationResult.Status.SERVICE_NOT_VERIFIED, ServiceCredentialsValidationResult.Status.SERVICE_NOT_VERIFIED.getMessage(), ex);
        } else {
          return new ServiceCredentialsValidationResult(ServiceCredentialsValidationResult.Status.CANT_CONNECT, ex.getMessage(), ex);
        }
      }
    } catch (OAuth2TokenLoadException e) {
      ServiceCredentialsValidationResult.Status status;
      String message;
      if (eq_sjvc5t_a0c0a01a6(e.getError(), OAuthTokenErrorCode.INVALID_CLIENT.getASCIICode())) {
        status = ServiceCredentialsValidationResult.Status.SERVICE_NOT_REGISTERED;
        message = "Service is not registered in Hub: " + e.getDescription();
      } else if (eq_sjvc5t_a0a2a0k0g(e.getError(), OAuthTokenErrorCode.INVALID_GRANT.getASCIICode())) {
        status = ServiceCredentialsValidationResult.Status.SERVICE_WRONG_SECRET;
        message = "Service secret is invalid: " + e.getDescription();
      } else if (eq_sjvc5t_a0b2a0k0g(e.getError(), OAuthTokenErrorCode.UNAUTHORIZED_CLIENT.getASCIICode())) {
        // JPS-758 
        status = ServiceCredentialsValidationResult.Status.SERVICE_NOT_VERIFIED;
        message = e.getDescription();
      } else {
        status = ServiceCredentialsValidationResult.Status.SERVICE_NOT_REGISTERED;
        message = e.getError() + ": " + e.getDescription();
      }
      return new ServiceCredentialsValidationResult(status, message, e);
    } catch (Exception e) {
      return new ServiceCredentialsValidationResult(ServiceCredentialsValidationResult.Status.CANT_CONNECT, e.getMessage(), e);
    }
  }

  @Nullable
  private ServiceJSON getCurrentService() {
    return getServiceClient().me(Partial.service(Partial.Service.ID, Partial.Service.TRUSTED));
  }

  @NotNull
  public Client getClient() {
    return getHubClient().getJerseyClient();
  }

  @Nullable
  public Integer getConnectTimeout() {
    return ((Integer) getClient().getConfiguration().getProperty(ClientProperties.CONNECT_TIMEOUT));
  }

  @Nullable
  public Integer getReadTimeout() {
    return ((Integer) getClient().getConfiguration().getProperty(ClientProperties.READ_TIMEOUT));
  }

  @NotNull
  public EventClient getEventClient() {
    return new EventClient(getRootResource(), getTokenHolder());
  }

  @NotNull
  public UserClient getUserClient() {
    return new UserClient(getRootResource(), getTokenHolder());
  }

  @NotNull
  public UserGroupClient getUserGroupClient() {
    return new UserGroupClient(getRootResource(), getTokenHolder());
  }

  @NotNull
  public ServiceClient getServiceClient() {
    return new ServiceClient(getRootResource(), getTokenHolder());
  }

  @NotNull
  public SettingClient getSettingsClient() {
    return new SettingClient(getRootResource(), getTokenHolder());
  }

  @NotNull
  public RoleClient getRoleClient() {
    return new RoleClient(getRootResource(), getTokenHolder());
  }

  @NotNull
  public AuthModuleClient getAuthModuleClient() {
    return new AuthModuleClient(getRootResource(), getTokenHolder());
  }

  @NotNull
  @Deprecated
  public SpaceClient getSpaceClient() {
    return new SpaceClient(getRootResource(), getTokenHolder());
  }

  @NotNull
  public ProjectClient getProjectClient() {
    return new ProjectClient(getRootResource(), getTokenHolder());
  }

  @NotNull
  public ResourceClient getResourceClient() {
    return new ResourceClient(getRootResource(), getTokenHolder());
  }

  @NotNull
  public KeyStoreClient getStoreClient() {
    return new KeyStoreClient(getRootResource(), getTokenHolder());
  }

  @NotNull
  public CertificateClient getCertificateClient() {
    return new CertificateClient(getRootResource(), getTokenHolder());
  }

  @NotNull
  public MetricClient getMetricsClient() {
    return new MetricClient(getRootResource(), getTokenHolder());
  }

  @NotNull
  public PermissionClient getPermissionClient() {
    return new PermissionClient(getRootResource(), getTokenHolder());
  }

  @NotNull
  @Deprecated
  public SpaceRoleClient getSpaceRoleClient() {
    return new SpaceRoleClient(getRootResource(), getTokenHolder());
  }

  @NotNull
  public ProjectRoleClient getProjectRoleClient() {
    return new ProjectRoleClient(getRootResource(), getTokenHolder());
  }

  @NotNull
  public UserDetailClient getUserDetailClient() {
    return new UserDetailClient(getRootResource(), getTokenHolder());
  }

  @NotNull
  public ImportClient getImportClient() {
    return new ImportClient(getRootResource(), getTokenHolder());
  }

  @Nullable
  public UserJSON getGuest(Partial<Partial.User> partial) {
    UserClient userClient = getUserClient();
    return userClient.guest(partial);
  }

  @NotNull
  @Deprecated
  public Iterable<SpaceJSON> getAccessibleSpaces(UserJSON user, PermissionJSON permission, Partial<Partial.Space> partial) {
    SpaceClient spaceClient = getSpaceClient();
    String query = String.format("accessible(for: {%s}, with: {%s})", user.getId(), permission.getId());
    SpaceClient.Page spacePage = spaceClient.getSpacePage(SpaceClient.filter().top(-1).query(query), partial);
    return spacePage.getSpaces();
  }

  @NotNull
  public HubURLBuilder getUrlBuilder() {
    return getHubClient().getURLBuilder();
  }

  private static boolean eq_sjvc5t_a0a0b0a0a3a01a6(Object a, Object b) {
    return (a != null ?
      a.equals(b) :
      a == b
    );
  }

  private static boolean eq_sjvc5t_a0a0a1a0a0d0k0g(Object a, Object b) {
    return (a != null ?
      a.equals(b) :
      a == b
    );
  }

  private static boolean eq_sjvc5t_a0c0a01a6(Object a, Object b) {
    return (a != null ?
      a.equals(b) :
      a == b
    );
  }

  private static boolean eq_sjvc5t_a0a2a0k0g(Object a, Object b) {
    return (a != null ?
      a.equals(b) :
      a == b
    );
  }

  private static boolean eq_sjvc5t_a0b2a0k0g(Object a, Object b) {
    return (a != null ?
      a.equals(b) :
      a == b
    );
  }
}
