package jetbrains.jetpass.client.oauth2;

/*Generated by MPS */

import java.net.URI;
import javax.ws.rs.client.Client;
import org.jetbrains.annotations.NotNull;
import jetbrains.jetpass.client.oauth2.token.OAuth2ClientFlow;
import jetbrains.jetpass.client.oauth2.auth.OAuth2CodeFlow;
import jetbrains.jetpass.client.oauth2.auth.OAuth2ImplicitFlow;
import jetbrains.jetpass.client.oauth2.token.OAuth2ResourceOwnerFlow;
import jetbrains.jetpass.client.oauth2.token.OAuth2RefreshTokenFlow;
import javax.ws.rs.WebApplicationException;
import javax.ws.rs.client.ClientBuilder;
import javax.ws.rs.core.UriBuilder;
import com.intellij.hub.core.URIHelper;

public class OAuth2Client {
  private final URI authServerURI;
  private final Client client;

  private OAuth2Client(@NotNull final Client client, @NotNull final URI authServerURI) {
    this.client = client;
    this.authServerURI = authServerURI;
  }

  @NotNull
  public OAuth2ClientFlow.Builder clientFlow() {
    return new OAuth2ClientFlow.Builder(client, authServerURI);
  }

  @NotNull
  public OAuth2CodeFlow.Builder codeFlow() {
    return new OAuth2CodeFlow.Builder(client, authServerURI);
  }

  @NotNull
  public OAuth2ImplicitFlow.Builder implicitFlow() {
    return new OAuth2ImplicitFlow.Builder(client, authServerURI);
  }

  @NotNull
  public OAuth2ResourceOwnerFlow.Builder resourceOwnerFlow() {
    return new OAuth2ResourceOwnerFlow.Builder(client, authServerURI);
  }

  @NotNull
  public OAuth2RefreshTokenFlow.Builder refreshTokenFlow() {
    return new OAuth2RefreshTokenFlow.Builder(client, authServerURI);
  }

  @NotNull
  public String getHubScope() throws WebApplicationException {
    return client.target(authServerURI).path(OAuth2Path.HUB_SCOPE.getPath()).request().get(String.class);
  }

  @NotNull
  public String getPublicKey() throws WebApplicationException {
    return client.target(authServerURI).path(OAuth2Path.PUBLIC_KEY.getPath()).request().get(String.class);

  }

  @NotNull
  public static OAuth2Client.OAuth2ClientBuilder builder(@NotNull Client client) {
    return new OAuth2Client.OAuth2ClientBuilder(client);
  }

  @NotNull
  public static OAuth2Client.OAuth2ClientBuilder builder() {
    return builder(ClientBuilder.newClient());
  }

  public static class OAuth2ClientBuilder {
    private final Client client;
    private URI authServerURI;

    private OAuth2ClientBuilder(@NotNull Client client) {
      this.client = client;
    }

    public OAuth2Client.OAuth2ClientBuilder baseUrl(@NotNull String baseUri) {
      return baseUrl(URI.create(baseUri));
    }

    public OAuth2Client.OAuth2ClientBuilder baseUrl(@NotNull URI baseUri) {
      return authServerURI(UriBuilder.fromUri(URIHelper.toApiUri(baseUri)).path("rest").path("oauth2").build());
    }

    public OAuth2Client.OAuth2ClientBuilder authServerURI(@NotNull String authServerURI) {
      return authServerURI(URI.create(authServerURI));
    }

    public OAuth2Client.OAuth2ClientBuilder authServerURI(@NotNull URI authServerURI) {
      this.authServerURI = authServerURI;
      return this;
    }

    @NotNull
    public Client getClient() {
      return this.client;
    }

    @NotNull
    public OAuth2Client build() {
      return new OAuth2Client(client, authServerURI);
    }
  }
}
