package jetbrains.jetpass.client.oauth2;

/*Generated by MPS */

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import jetbrains.jetpass.client.oauth2.auth.OAuth2AuthFlow;
import javax.ws.rs.client.Client;
import java.net.URI;
import javax.ws.rs.core.MultivaluedMap;
import com.intellij.hub.auth.oauth2.token.ScopeUtil;
import javax.ws.rs.core.UriBuilder;
import java.util.List;
import java.util.ArrayList;
import java.util.Map;
import com.intellij.hub.auth.request.TokenFormParameter;
import java.util.Set;
import java.util.HashSet;
import org.jetbrains.annotations.NotNull;
import org.jetbrains.annotations.Nullable;
import java.util.Arrays;

public abstract class OAuth2Flow {
  private static final Logger log = LoggerFactory.getLogger(OAuth2AuthFlow.class);

  protected final Client client;
  protected final URI authServerURI;
  protected final String clientId;
  protected final String[] scope;
  protected final String[] encodedScope;

  protected OAuth2Flow(OAuth2Flow.Builder<? extends OAuth2Flow.Builder, ? extends OAuth2Flow> builder) {
    this.client = builder.client;
    this.clientId = builder.clientId;
    this.scope = builder.scope.toArray(new String[builder.scope.size()]);
    this.encodedScope = builder.encodedScope.toArray(new String[builder.encodedScope.size()]);
    this.authServerURI = builder.authServerURI;

  }

  public String getClientServiceId() {
    return clientId;
  }

  protected static void param(MultivaluedMap<String, String> params, String key, String value) {
    if (value != null) {
      params.putSingle(key, value);
    }
  }

  protected static void scope(MultivaluedMap<String, String> params, String key, String[] scope, String[] encodedScopes) {
    StringBuilder resultScopes = new StringBuilder();
    boolean first = true;
    if (scope != null && scope.length > 0) {
      for (String s : scope) {
        if (first) {
          first = false;
        } else {
          resultScopes.append(ScopeUtil.SCOPE_DELIMITER);
        }
        resultScopes.append(ScopeUtil.encode(s));
      }
    }
    if (encodedScopes != null && encodedScopes.length > 0) {
      for (String s : encodedScopes) {
        if (first) {
          first = false;
        } else {
          resultScopes.append(ScopeUtil.SCOPE_DELIMITER);
        }
        resultScopes.append(s);
      }
    }

    if (resultScopes.length() > 0) {
      params.putSingle(key, resultScopes.toString());
    }
  }

  protected static URI queryParams(UriBuilder uriBuilder, MultivaluedMap<String, String> params) {
    OAuth2Flow.logRequest(uriBuilder, params);

    List<String> values = new ArrayList<String>(params.size());
    for (String paramName : params.keySet()) {
      List<String> paramValues = params.get(paramName);
      if (paramValues != null && !(paramValues.isEmpty())) {
        String[] placeholderArray = new String[paramValues.size()];
        for (int i = 0; i < paramValues.size(); i++) {
          String placeholder = "{p" + values.size() + "}";
          placeholderArray[i] = placeholder;
          values.add(paramValues.get(i));
        }
        uriBuilder.queryParam(paramName, placeholderArray);
      }
    }
    return uriBuilder.build(values.toArray());
  }

  public static void logRequest(UriBuilder resource, Map<String, List<String>> params) {
    if (OAuth2Flow.log.isDebugEnabled()) {
      OAuth2Flow.log.debug("Request to: " + resource.build());
      StringBuilder out = new StringBuilder("Params: {\n");
      boolean first = true;
      for (String key : params.keySet()) {
        if (!(first)) {
          out.append("\n");
        }
        first = false;
        StringBuilder value = out.append("\t").append(key).append(": ");
        if (key.equals(TokenFormParameter.PASSWORD)) {
          value.append("********");
        } else {
          value.append(params.get(key));
        }
      }
      OAuth2Flow.log.debug(out.toString());
    }
  }

  public static abstract class Builder<T extends OAuth2Flow.Builder<T, F>, F extends OAuth2Flow> {
    private final Client client;
    private URI authServerURI;
    private Set<String> scope = new HashSet<String>(1);
    private Set<String> encodedScope = new HashSet<String>(1);
    private String clientId;

    protected Builder(@NotNull Client client, @NotNull URI authServerURI) {
      this.client = client;
      this.authServerURI = authServerURI;
    }

    private T _this() {
      return (T) this;
    }

    @NotNull
    public T scope(@Nullable String... scope) {
      if (scope != null) {
        this.scope = new HashSet<String>(Arrays.asList(scope));
      }
      return _this();
    }

    @NotNull
    public T addScope(@Nullable String scope) {
      if (scope != null) {
        this.scope.add(scope);
      }
      return _this();
    }

    @NotNull
    public T addEncodedScope(@Nullable String scope) {
      if (scope != null) {
        this.encodedScope.add(scope);
      }
      return _this();
    }

    @NotNull
    public T clientId(@NotNull String clientId) {
      this.clientId = clientId;
      return _this();
    }

    public abstract F build();
  }
}
