package jetbrains.jetpass.client.oauth2;

/*Generated by MPS */

import com.intellij.hub.auth.oauth2.consumer.TokenLoader;
import javax.ws.rs.client.WebTarget;
import javax.ws.rs.core.MultivaluedMap;
import jetbrains.jetpass.client.oauth2.token.OAuth2TokenFlow;
import org.jetbrains.annotations.NotNull;
import com.intellij.hub.auth.basic.BasicAuthToken;
import com.intellij.hub.auth.oauth2.consumer.OAuth2TokenLoadException;
import javax.ws.rs.client.Invocation;
import javax.ws.rs.core.MediaType;
import com.intellij.hub.auth.request.Header;
import com.intellij.hub.auth.request.TokenFormParameter;
import javax.ws.rs.core.Response;
import javax.ws.rs.client.Entity;
import jetbrains.jetpass.rest.dto.ErrorJSON;
import jetbrains.jetpass.oauth2.rest.dto.AccessTokenJSON;

public class SimpleTokenLoader implements TokenLoader {
  private final WebTarget tokenResource;
  private final MultivaluedMap<String, String> params;
  private final OAuth2TokenFlow.ClientAuth clientAuth;

  public SimpleTokenLoader(WebTarget tokenResouce, MultivaluedMap<String, String> params, OAuth2TokenFlow.ClientAuth clientAuth) {
    this.tokenResource = tokenResouce;
    this.params = params;
    this.clientAuth = clientAuth;
  }

  @NotNull
  public TokenLoader.TokenData loadAccessToken(@NotNull final BasicAuthToken auth) throws OAuth2TokenLoadException {
    Invocation.Builder request = tokenResource.request(MediaType.APPLICATION_JSON_TYPE);

    switch (clientAuth) {
      case HEADER:
        Header basicAuthHeader = Header.authorization(auth);
        request.header(basicAuthHeader.getName(), basicAuthHeader.getValue());
        break;
      case FORM:
        params.putSingle(TokenFormParameter.CLIENT_ID, auth.getUsername());
        params.putSingle(TokenFormParameter.CLIENT_SECRET, auth.getPassword());
        break;
      default:
    }

    OAuth2Flow.logRequest(tokenResource.getUriBuilder(), params);

    long now = System.currentTimeMillis();
    Response response = request.post(Entity.form(params), Response.class);

    if (!(Response.Status.OK.getStatusCode() == response.getStatus())) {
      ErrorJSON error;
      try {
        error = response.readEntity(ErrorJSON.class);
      } catch (Exception e) {
        throw new OAuth2TokenLoadException("unknown_error", response.getStatus(), e.getMessage(), null, e);
      }
      throw new OAuth2TokenLoadException(error.getError(), error.getError_code(), error.getError_description(), error.getError_uri());
    }

    AccessTokenJSON token = response.readEntity(AccessTokenJSON.class);

    return new TokenLoader.TokenData(token.getAccess_token(), token.getRefresh_token(), token.getExpires_in() * 1000 + now);
  }
}
