package jetbrains.jetpass.client.oauth2.auth;

/*Generated by MPS */

import jetbrains.jetpass.client.oauth2.OAuth2Flow;
import com.intellij.hub.auth.request.AuthRequestParameter;
import java.net.URI;
import org.jetbrains.annotations.NotNull;
import javax.ws.rs.core.UriBuilder;
import jetbrains.jetpass.client.oauth2.OAuth2Path;
import javax.ws.rs.core.MultivaluedMap;
import javax.ws.rs.core.MultivaluedHashMap;
import javax.ws.rs.client.Client;
import org.jetbrains.annotations.Nullable;
import com.intellij.hub.auth.request.ApprovalPromptValue;

public abstract class OAuth2AuthFlow extends OAuth2Flow {
  protected final AuthRequestParameter.RequestCredentials credentials;
  protected final String approvalPrompt;
  protected final String responseType;
  protected final String state;
  protected final String message;
  protected final URI redirectUri;

  protected OAuth2AuthFlow(OAuth2AuthFlow.Builder<? extends OAuth2AuthFlow.Builder, ? extends OAuth2AuthFlow> builder, String responseType) {
    super(builder);
    credentials = builder.credentialsRequest;
    approvalPrompt = builder.approvalPromptValue;
    state = builder.state;
    message = builder.message;
    redirectUri = builder.redirectUri;
    this.responseType = responseType;
  }

  @NotNull
  public URI getAuthUri() {
    UriBuilder authUri = UriBuilder.fromUri(authServerURI).path(OAuth2Path.AUTH.getPath());

    return queryParams(authUri, buildParams());
  }

  @NotNull
  protected MultivaluedMap<String, String> buildParams() {
    MultivaluedMap<String, String> params = new MultivaluedHashMap();
    param(params, AuthRequestParameter.RESPONSE_TYPE, responseType);
    param(params, AuthRequestParameter.CLIENT_ID, clientId);
    scope(params, AuthRequestParameter.SCOPE, scope, encodedScope);
    param(params, AuthRequestParameter.MESSAGE, message);
    param(params, AuthRequestParameter.REDIRECT_URI, (redirectUri == null ?
      null :
      redirectUri.toASCIIString()
    ));
    param(params, AuthRequestParameter.STATE, state);
    param(params, AuthRequestParameter.APPROVAL_PROMPT, approvalPrompt);
    param(params, AuthRequestParameter.REQUEST_CREDENTIALS, (credentials == null ?
      null :
      credentials.name().toLowerCase()
    ));
    return params;
  }

  public static abstract class Builder<B extends OAuth2AuthFlow.Builder<B, Flow>, Flow extends OAuth2AuthFlow> extends OAuth2Flow.Builder<B, Flow> {
    private AuthRequestParameter.RequestCredentials credentialsRequest;
    private String approvalPromptValue;
    private String state;
    private String message;
    private URI redirectUri;

    protected Builder(@NotNull Client client, @NotNull URI authServerURI) {
      super(client, authServerURI);
    }

    @NotNull
    public B credentials(@Nullable AuthRequestParameter.RequestCredentials credentialsRequest) {
      this.credentialsRequest = credentialsRequest;
      return ((B) this);
    }

    @NotNull
    public B state(@Nullable String state) {
      this.state = state;
      return ((B) this);
    }

    @NotNull
    public B message(@Nullable String message) {
      this.message = message;
      return ((B) this);
    }

    @NotNull
    public B approval(@Nullable String approvalPromptValue) {
      String value = null;
      if (ApprovalPromptValue.AUTO.equalsIgnoreCase(approvalPromptValue)) {
        value = ApprovalPromptValue.AUTO;
      } else if (ApprovalPromptValue.FORCE.equalsIgnoreCase(approvalPromptValue)) {
        value = ApprovalPromptValue.FORCE;
      }
      this.approvalPromptValue = value;
      return ((B) this);
    }

    @NotNull
    public B redirectUri(@Nullable URI redirectUri) {
      this.redirectUri = redirectUri;
      return ((B) this);
    }

    @NotNull
    public B redirectUri(@Nullable String redirectUri) {
      return redirectUri((redirectUri != null ?
        URI.create(redirectUri) :
        null
      ));
    }
  }
}
