package jetbrains.jetpass.client.oauth2.token;

/*Generated by MPS */

import java.net.URI;
import javax.ws.rs.core.MultivaluedMap;
import javax.ws.rs.core.MultivaluedHashMap;
import com.intellij.hub.auth.request.TokenFormParameter;
import com.intellij.hub.auth.request.GrantType;
import com.intellij.hub.auth.oauth2.consumer.TokenLoader;
import org.jetbrains.annotations.NotNull;
import com.intellij.hub.auth.oauth2.consumer.OAuth2TokenLoadException;
import com.intellij.hub.auth.oauth2.error.OAuthTokenErrorCode;
import org.jetbrains.annotations.Nullable;
import javax.ws.rs.client.Client;

public class OAuth2CodeResponseFlow extends OAuth2TokenFlow {
  private final String code;
  private final URI redirectUri;
  private volatile String refreshToken = null;

  private OAuth2CodeResponseFlow(OAuth2CodeResponseFlow.Builder builder) {
    super(builder);
    this.code = builder.code;
    redirectUri = builder.redirectUri;
  }

  protected MultivaluedMap<String, String> buildParams() {
    MultivaluedMap mMap = new MultivaluedHashMap();
    param(mMap, TokenFormParameter.GRANT_TYPE, GrantType.AUTHORIZATION_CODE);
    param(mMap, TokenFormParameter.CODE, code);
    param(mMap, TokenFormParameter.REDIRECT_URI, (redirectUri == null ?
      null :
      redirectUri.toASCIIString()
    ));
    scope(mMap, TokenFormParameter.SCOPE, scope, encodedScope);
    return mMap;
  }

  @Override
  protected TokenLoader.TokenData refresh(@NotNull final TokenLoader.TokenData data) {
    String token = data.getRefreshToken();
    if ((token != null && token.length() > 0)) {
      refreshToken = token;
      return getRefreshTokenFlow(token).getTokenData();
    }
    throw new OAuth2TokenLoadException(OAuthTokenErrorCode.INVALID_GRANT.createError("Unable to refresh expired token"));
  }

  @Nullable
  public OAuth2RefreshTokenFlow getRefreshTokenFlow() {
    return getRefreshTokenFlow(getRefreshToken());
  }

  @Nullable
  public String getRefreshToken() {
    return getRefreshToken(getTokenData());
  }

  @Nullable
  private String getRefreshToken(@NotNull final TokenLoader.TokenData currentData) {
    String currentRefreshToken = refreshToken;
    if (currentRefreshToken == null) {
      currentRefreshToken = currentData.getRefreshToken();
      if (currentRefreshToken != null) {
        refreshToken = currentRefreshToken;
      }
    }
    return currentRefreshToken;
  }

  @Nullable
  private OAuth2RefreshTokenFlow getRefreshTokenFlow(@Nullable String token) {
    return (token != null ?
      appendSecret(new OAuth2RefreshTokenFlow.Builder(client, authServerURI).clientId(clientId)).token(token).build() :
      null
    );
  }

  public static class Builder extends OAuth2TokenFlow.Builder<OAuth2CodeResponseFlow.Builder, OAuth2CodeResponseFlow> {
    private String code;
    private URI redirectUri;

    public Builder(@NotNull Client client, @NotNull URI authServerURI) {
      super(client, authServerURI);
    }

    @NotNull
    public OAuth2CodeResponseFlow.Builder code(@Nullable String code) {
      this.code = code;
      return this;
    }

    @NotNull
    public OAuth2CodeResponseFlow.Builder redirectUri(@Nullable URI redirectUri) {
      this.redirectUri = redirectUri;
      return this;
    }

    @NotNull
    public OAuth2CodeResponseFlow.Builder redirectUri(@Nullable String redirectUri) {
      return redirectUri((redirectUri != null ?
        URI.create(redirectUri) :
        null
      ));
    }

    @NotNull
    public OAuth2CodeResponseFlow build() {
      return new OAuth2CodeResponseFlow(this);
    }
  }
}
