package jetbrains.jetpass.client.oauth2.token;

/*Generated by MPS */

import jetbrains.jetpass.client.oauth2.OAuth2Flow;
import com.intellij.hub.auth.oauth2.consumer.TokenHolder;
import com.intellij.hub.auth.oauth2.token.AccessToken;
import org.jetbrains.annotations.Nullable;
import org.jetbrains.annotations.NotNull;
import java.util.concurrent.atomic.AtomicReference;
import com.intellij.hub.auth.oauth2.consumer.TokenLoader;
import javax.ws.rs.client.WebTarget;
import jetbrains.jetpass.client.oauth2.OAuth2Path;
import javax.ws.rs.core.MultivaluedMap;
import jetbrains.jetpass.client.oauth2.SimpleTokenLoader;
import com.intellij.hub.auth.oauth2.consumer.OAuth2TokenLoadException;
import com.intellij.hub.auth.basic.BasicAuthToken;
import com.intellij.hub.auth.request.Header;
import javax.ws.rs.client.Client;
import java.net.URI;

public abstract class OAuth2TokenFlow extends OAuth2Flow implements TokenHolder<AccessToken> {
  @Nullable
  private final String clientSecret;
  @NotNull
  private final OAuth2TokenFlow.ClientAuth clientAuth;
  @NotNull
  private final AtomicReference<TokenLoader.TokenData> cache = new AtomicReference<TokenLoader.TokenData>();

  protected OAuth2TokenFlow(OAuth2TokenFlow.Builder<? extends OAuth2TokenFlow.Builder, ? extends OAuth2TokenFlow> builder) {
    super(builder);
    this.clientSecret = builder.clientSecret;
    this.clientAuth = builder.clientAuth;
  }

  @NotNull
  public TokenLoader getLoader() {
    WebTarget res = client.target(authServerURI).path(OAuth2Path.TOKEN.getPath());
    MultivaluedMap params = buildParams();

    return new SimpleTokenLoader(res, params, clientAuth);
  }

  @NotNull
  public AccessToken getToken() throws OAuth2TokenLoadException {
    return getTokenData().getAccessToken();
  }

  @NotNull
  public final TokenLoader.TokenData getToken(@NotNull final BasicAuthToken token) throws OAuth2TokenLoadException {
    return new OAuth2ResourceOwnerFlow.Builder(client, authServerURI).scope(scope).clientSecret(clientSecret).clientId(clientId).clientAuth(clientAuth).username(token.getUsername()).password(token.getPassword()).build().getTokenData();
  }

  @NotNull
  public String getRawToken() throws OAuth2TokenLoadException {
    return getTokenData().getRawAccessToken();
  }

  protected final <B extends OAuth2TokenFlow.Builder<B, F>, F extends OAuth2TokenFlow> B appendSecret(B builder) {
    // I don't want to expose clientSecret in any form 
    return builder.clientSecret(clientSecret).clientAuth(clientAuth);
  }

  @NotNull
  protected final TokenLoader.TokenData getTokenData() throws OAuth2TokenLoadException {
    TokenLoader.TokenData current = cache.get();
    TokenLoader.TokenData tokenData = current;
    if (tokenData == null) {
      tokenData = getLoader().loadAccessToken(new BasicAuthToken(clientId, clientSecret));
    } else if (tokenData.isExpired()) {
      tokenData = refresh(tokenData);
    } else {
      return tokenData;
    }

    if (cache.compareAndSet(current, tokenData)) {
      return tokenData;
    } else {
      // Smb. set new token before us. try once more. 
      return getTokenData();
    }
  }

  protected TokenLoader.TokenData refresh(@NotNull final TokenLoader.TokenData data) {
    return getLoader().loadAccessToken(new BasicAuthToken(clientId, clientSecret));
  }

  @NotNull
  public Header getHeader() throws OAuth2TokenLoadException {
    return Header.authorization(getToken());
  }

  protected abstract MultivaluedMap<String, String> buildParams();

  public static abstract class Builder<B extends OAuth2TokenFlow.Builder<B, Flow>, Flow extends OAuth2TokenFlow> extends OAuth2Flow.Builder<B, Flow> {
    private String clientSecret;
    @NotNull
    private OAuth2TokenFlow.ClientAuth clientAuth = OAuth2TokenFlow.ClientAuth.HEADER;

    protected Builder(@NotNull Client client, @NotNull URI authServerURI) {
      super(client, authServerURI);
    }

    @NotNull
    public B clientSecret(@Nullable String serviceSecret) {
      this.clientSecret = serviceSecret;
      return ((B) this);
    }

    @NotNull
    public B clientAuth(@NotNull OAuth2TokenFlow.ClientAuth clientAuth) {
      this.clientAuth = clientAuth;
      return ((B) this);
    }
  }

  public static   enum ClientAuth {
    HEADER(),
    FORM();

    ClientAuth() {
    }
  }
}
