package jetbrains.jetpass.client.resource;

/*Generated by MPS */

import com.intellij.hub.core.Error;
import javax.ws.rs.WebApplicationException;
import javax.ws.rs.core.Response;
import jetbrains.jetpass.rest.dto.ErrorJSON;
import org.jetbrains.annotations.NotNull;
import org.jetbrains.annotations.Nullable;
import java.net.URI;


/**
 * Error helper returns full info about WebApplicationException in HUB Common Error format
 */
public class ErrorHelper {
  public static Error readError(WebApplicationException ex) {
    Response response = ex.getResponse();
    Error error = new ErrorHelper.BasicAPIError(response.getStatusInfo(), ex.getMessage());

    try {
      ErrorJSON json = response.readEntity(ErrorJSON.class);
      error = new ErrorHelper.JSONAPIError(json);
    } catch (Throwable t) {
      // Ignore 
    }

    return error;
  }

  private static class BasicAPIError implements Error {
    private final Response.StatusType type;
    private final String message;

    public BasicAPIError(@NotNull Response.StatusType type, @NotNull String message) {
      this.type = type;
      this.message = message;
    }

    @NotNull
    public String getError() {
      return type.getReasonPhrase();
    }

    public int getStatusCode() {
      return type.getStatusCode();
    }

    @NotNull
    public String getMessage() {
      return message;
    }

    @Nullable
    public String getDeveloperMessage() {
      return null;
    }

    @Nullable
    public URI getURI() {
      return null;
    }

    @Nullable
    public String getField() {
      return null;
    }
  }

  private static class JSONAPIError implements Error {
    private final ErrorJSON json;

    public JSONAPIError(ErrorJSON json) {
      this.json = json;
    }

    @NotNull
    public String getError() {
      return json.getError();
    }

    public int getStatusCode() {
      return json.getError_code();
    }

    @NotNull
    public String getMessage() {
      return json.getError_description();
    }

    @Nullable
    public String getDeveloperMessage() {
      return json.getError_developer_message();
    }

    @Nullable
    public URI getURI() {
      return URI.create(json.getError_uri());
    }

    @Nullable
    public String getField() {
      return json.getError_field();
    }
  }
}
