package jetbrains.jetpass.dao.api.permissionCache;

/*Generated by MPS */

import jetbrains.jetpass.api.authority.UserGroup;
import jetbrains.jetpass.api.security.ProjectRole;
import jetbrains.jetpass.api.authority.User;
import java.util.Iterator;
import java.util.Map;
import java.util.List;
import java.util.HashMap;
import java.util.ArrayList;
import java.util.Collections;

public class Handler {
  private static final String[] EMPTY_GROUP_UUIDS = new String[0];
  private static final String[][] EMPTY_PROJECTROLE_UUIDS = new String[0][0];

  private final String[] groupUUIDs;
  private final String[][] projectRolesUUIDs;

  public Handler(final String[] groupUUIDs, final String[][] projectRoleUUIDs) {
    this.groupUUIDs = groupUUIDs;
    this.projectRolesUUIDs = projectRoleUUIDs;
  }

  public Handler(Iterable<? extends UserGroup> groups, Iterable<? extends ProjectRole> projectRoles, StringInterner interner) {
    this(toGroupUUIDS(groups, interner), toProjectRoleUUIDs(projectRoles, interner));
  }

  public Handler(final User user, StringInterner interner) {
    this(user.getGroups(), user.getProjectRoles(), interner);
  }

  public String[] getGroupUUIDs() {
    return (groupUUIDs == null ?
      EMPTY_GROUP_UUIDS :
      groupUUIDs
    );
  }

  @Deprecated
  public String[][] getSpaceRolesUUIDs() {
    return getProjectRolesUUIDs();
  }

  public String[][] getProjectRolesUUIDs() {
    return (projectRolesUUIDs == null ?
      EMPTY_PROJECTROLE_UUIDS :
      projectRolesUUIDs
    );
  }

  @Override
  public boolean equals(Object object) {
    if (this == object) {
      return true;
    }
    if (object == null) {
      return false;
    }
    if (!(object instanceof Handler)) {
      return false;
    }

    Handler that = (Handler) object;
    return this.areGroupsEqual(that) && this.areOwnRolesEqual(that);
  }

  private boolean areGroupsEqual(Handler that) {
    if (this.groupUUIDs == that.groupUUIDs) {
      return true;
    }
    if (this.groupUUIDs == null || that.groupUUIDs == null) {
      return false;
    }
    if (this.groupUUIDs.length != that.groupUUIDs.length) {
      return false;
    }
    for (int i = 0; i < groupUUIDs.length; i++) {
      // UUIDs can be interned by different interner instances 
      if (!(this.groupUUIDs[i].equals(that.groupUUIDs[i]))) {
        return false;
      }
    }
    return true;
  }

  private boolean areOwnRolesEqual(Handler that) {
    if (this.projectRolesUUIDs == that.projectRolesUUIDs) {
      return true;
    }
    if (this.projectRolesUUIDs == null || that.projectRolesUUIDs == null) {
      return false;
    }
    if (this.projectRolesUUIDs.length != that.projectRolesUUIDs.length) {
      return false;
    }
    for (int i = 0; i < projectRolesUUIDs.length; i++) {
      // UUIDs can be interned by different interner instances 
      if (!(this.projectRolesUUIDs[i][0].equals(that.projectRolesUUIDs[i][0]) && this.projectRolesUUIDs[i][1].equals(that.projectRolesUUIDs[i][1]))) {
        return false;
      }
    }
    return true;
  }

  @Override
  public int hashCode() {
    int hash = 0;
    if (groupUUIDs != null) {
      for (String uuid : groupUUIDs) {
        hash = hash ^ uuid.hashCode();
      }
    }
    if (projectRolesUUIDs != null) {
      for (String[] uuid : projectRolesUUIDs) {
        hash = hash ^ uuid[0].hashCode() ^ uuid[1].hashCode();
      }
    }
    return hash;
  }

  /**
   * Converts projectRole sequence to 2 dimensional array of projectRoleUUID (interned)
   * BC! Doesn't contain included roles
   * Where arr[i][0] - project, arr[i][1] - role
   * 
   * @param projectRoles sequence of project roles
   * @return arr[i][0] - project, arr[i][1] - role
   */
  protected static String[][] toProjectRoleUUIDs(Iterable<? extends ProjectRole> projectRoles, StringInterner interner) {
    String[][] projectRolesUUIDs = null;
    Iterator<? extends ProjectRole> iter = projectRoles.iterator();
    if (iter.hasNext()) {
      Map<String, List<String>> projects = new HashMap<String, List<String>>();
      int projectRoleCount = 0;
      for (ProjectRole projectRole; iter.hasNext();) {
        projectRole = iter.next();
        projectRoleCount++;
        String projectUUID = interner.intern(projectRole.getProject().getId());
        List<String> roles = projects.get(projectUUID);
        if (roles == null) {
          roles = new ArrayList<String>();
          projects.put(projectUUID, roles);
        }
        roles.add(interner.intern(projectRole.getRole().getId()));
      }

      List<String> sortedProjects = new ArrayList<String>(projects.keySet());
      Collections.sort(sortedProjects);

      List<String[]> pairs = new ArrayList<String[]>();
      for (String projectUUID : sortedProjects) {
        List<String> roles = projects.get(projectUUID);
        Collections.sort(roles);
        for (String rolesUUID : roles) {
          pairs.add(new String[]{projectUUID, rolesUUID});
        }
      }

      projectRolesUUIDs = pairs.toArray(new String[projectRoleCount][2]);
    }
    return projectRolesUUIDs;
  }

  /**
   * Converts userGroups sequence to array of userGroupsUUIDs (interned)
   * 
   * @param groups sequence of groups
   * @return arr[i] - groupUUID
   */
  protected static String[] toGroupUUIDS(Iterable<? extends UserGroup> groups, StringInterner interner) {
    String[] groupUUIDs = null;
    Iterator<? extends UserGroup> iterator = groups.iterator();
    if (iterator.hasNext()) {
      List<String> uuids = new ArrayList<String>();
      for (; iterator.hasNext();) {
        uuids.add(interner.intern(iterator.next().getId()));
      }
      Collections.sort(uuids);
      groupUUIDs = uuids.toArray(new String[uuids.size()]);
    }
    return groupUUIDs;
  }
}
