package jetbrains.jetpass.dao.api.permissionCache;

/*Generated by MPS */

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import jetbrains.jetpass.dao.api.authority.UserDAO;
import jetbrains.jetpass.dao.api.authority.UserGroupDAO;
import jetbrains.jetpass.dao.api.ServiceDAO;
import jetbrains.jetpass.dao.api.security.ProjectDAO;
import jetbrains.jetpass.dao.api.security.ResourceDAO;
import jetbrains.jetpass.dao.api.security.PermissionDAO;
import java.util.List;
import java.util.ArrayList;
import jetbrains.jetpass.api.authority.AuthorityHolder;
import jetbrains.jetpass.dao.api.DAO;
import jetbrains.jetpass.api.security.Project;
import jetbrains.jetpass.api.security.Permission;
import jetbrains.jetpass.api.security.Space;
import jetbrains.jetpass.api.security.Resource;
import jetbrains.jetpass.sequence.Sequence;
import java.util.Map;
import jetbrains.jetpass.api.IdItem;
import java.util.HashMap;
import java.util.Comparator;

public class PermissionCache {
  private static final int DEFAULT_USERS_PACK_SIZE = 2000;
  protected static Log log = LogFactory.getLog(PermissionCache.class);

  private volatile PermissionCache.Data data = new PermissionCache.Data();
  private volatile boolean canceled;
  private volatile boolean suspended;
  private volatile boolean dirty = true;
  private UserDAO myUserDAO;
  private UserGroupDAO myGroupDAO;
  private ServiceDAO myServiceDAO;
  private ProjectDAO myProjectDAO;
  private ResourceDAO myResourceDAO;
  private PermissionDAO myPermissionDAO;
  private PrivilegedRunner myPrivilegedRunner;
  private List<PermissionCacheListener> myListeners;
  private int myUsersPackSize;

  public PermissionCache() {
    PermissionCache.this.setUsersPackSize(DEFAULT_USERS_PACK_SIZE);
    PermissionCache.this.setListeners(new ArrayList<PermissionCacheListener>(2));
  }

  public PermissionQuery user(final String userUUID) {
    return user(userUUID, false);
  }

  public PermissionQuery user(final String userUUID, boolean dirty) {
    final StringInterner interner = new StringInterner();
    return query(userUUID, new PermissionCache.DataFinder.HandlerKey(this, userUUID, interner), PermissionCache.this.getUserDAO(), dirty, new PermissionCalculator.User(new PermissionCalculator.UserGroup(interner, PermissionData.createFactory(interner))));
  }

  public PermissionData getByHandler(final Handler handler) {
    return data.userPermissions.get(handler);
  }

  public PermissionQuery group(final String userGroupUUID) {
    return group(userGroupUUID, false);
  }

  public PermissionQuery group(final String userGroupUUID, boolean dirty) {
    StringInterner interner = new StringInterner();
    return query(userGroupUUID, new PermissionCache.DataFinder.StringKey(data.groupPermissions, userGroupUUID), PermissionCache.this.getGroupDAO(), dirty, new PermissionCalculator.UserGroup(interner, PermissionData.createFactory(interner)));
  }

  public PermissionQuery service(final String serviceUUID) {
    return service(serviceUUID, false);
  }

  public PermissionQuery service(final String serviceUUID, boolean dirty) {
    /*
      StringInterner interner = new StringInterner();
      query(serviceUUID, new PermissionCache.DataFinder.StringKey(data.servicePermissions, serviceUUID), PermissionCache.this.getServiceDAO(), dirty, new PermissionCalculator.Service(interner, PermissionData.createFactory(interner)));
    */
    final PermissionQuery[] result = new PermissionQuery[1];
    runPrivileged(new Runnable() {
      public void run() {
        result[0] = ServicePermissionQuery.create(PermissionCache.this.getServiceDAO(), serviceUUID, PermissionCache.this.getProjectDAO(), PermissionCache.this.getPermissionDAO());
      }
    });
    return result[0];
  }

  private <T extends AuthorityHolder, Key> PermissionQuery query(final String principalUUID, PermissionCache.DataFinder<Key> finder, final DAO<T> dao, boolean acceptDirty, final PermissionCalculator<T, Key, PermissionData> calculator) {
    boolean dirty = this.dirty;

    final PermissionData data;
    if (!(dirty) || acceptDirty) {
      PermissionData permissionData = finder.find();

      if (permissionData == null && dirty) {
        // If we accept dirty, but missed cache - build query not cached 
        data = calculate(principalUUID, dao, calculator);
      } else {
        data = (permissionData == null ?
          PermissionData.EMPTY :
          permissionData
        );
      }
    } else {
      data = calculate(principalUUID, dao, calculator);
    }
    return data.toQuery();
  }

  public <T extends AuthorityHolder, Key, Node extends IPermissionData> Node calculate(final String principalUUID, final DAO<T> dao, final PermissionCalculator<T, Key, Node> calculator) {
    final IPermissionData[] permissionQuery = new IPermissionData[1];
    runPrivileged(new Runnable() {
      public void run() {
        T principal = toPrincipal(dao, principalUUID);
        Project globalProject = PermissionCache.this.getProjectDAO().getGlobal();
        jetbrains.jetpass.api.authority.UserGroup allUsersGroup = PermissionCache.this.getGroupDAO().getRootUserGroup();
        permissionQuery[0] = calculator.calculate(principal, globalProject, allUsersGroup);
      }
    });
    return ((Node) permissionQuery[0]);
  }

  @Deprecated
  public boolean hasPermission(PermissionQuery query, Permission permission, Space space) {
    if (permission == null || isEmpty_dn6lih_a0a0a9(permission.getId())) {
      return false;
    }

    PermissionQuery permissionQuery = query.withPermission(permission.getId());
    if (hasGlobalPermission(permissionQuery, permission)) {
      return true;
    }

    return ((space != null ?
      permissionQuery.toProject(space.getId()) :
      permissionQuery.toGlobalProject()
    )).canAccess();
  }

  public boolean hasPermission(PermissionQuery query, Permission permission, Project project) {
    if (permission == null || isEmpty_dn6lih_a0a0a01(permission.getId())) {
      return false;
    }

    PermissionQuery permissionQuery = query.withPermission(permission.getId());
    if (hasGlobalPermission(permissionQuery, permission)) {
      return true;
    }

    return ((project != null ?
      permissionQuery.toProject(project.getId()) :
      permissionQuery.toGlobalProject()
    )).canAccess();
  }

  public boolean hasPermission(PermissionQuery query, Permission permission, Resource resource) {
    if (permission == null || isEmpty_dn6lih_a0a0a11(permission.getId())) {
      return false;
    }

    PermissionQuery permissionQuery = query.withPermission(permission.getId());

    // It is global permission, and user has somwhere 
    if (hasGlobalPermission(permissionQuery, permission)) {
      return true;
    }

    // Has permission in one of resource project. If resource is not in a single project then it's in Global 
    if (resource != null) {
      Sequence<Project> projects = this.getResourceDAO().getProjects(resource.getId());
      for (Project project : projects) {
        if (query.toProject(project.getId()).canAccess()) {
          return true;
        }
      }
    }

    // If we don't have access to particular project - check global 
    return query.toGlobalProject().canAccess();
  }

  private boolean hasGlobalPermission(PermissionQuery permissionQuery, Permission permission) {
    return permission.isGlobal() && permissionQuery.canAccess();
  }

  public void init() {
    runPrivileged(new Runnable() {
      public void run() {
        canceled = false;
        StringInterner interner = new StringInterner();
        PermissionData.Factory factory = PermissionData.createFactory(interner);
        final PermissionCalculator.Service serviceCalculator = new PermissionCalculator.Service(interner, factory);
        final PermissionCalculator.UserGroup groupCalculator = new PermissionCalculator.UserGroup(interner, factory);
        final PermissionCalculator.User userCaclulator = new PermissionCalculator.User(groupCalculator);

        double globalTime = start("Start (re)calculating permissions cache");
        Project global = PermissionCache.this.getProjectDAO().getGlobal();
        jetbrains.jetpass.api.authority.UserGroup allUsersGroup = PermissionCache.this.getGroupDAO().getRootUserGroup();

        double localTime = startPortion("Start recalculating service(s) permissions");
        for (jetbrains.jetpass.api.Service service : PermissionCache.this.getServiceDAO().getAllItems()) {
          if (checkCanceled(globalTime)) {
            return;
          }
          serviceCalculator.calculate(service, global, allUsersGroup);
        }
        finishPortion("Finish recalculating service(s) permissions in: %fs", localTime);

        if (checkCanceled(globalTime)) {
          return;
        }

        localTime = startPortion("Start recalculating usergroup(s) permissions");
        for (jetbrains.jetpass.api.authority.UserGroup group : PermissionCache.this.getGroupDAO().getAllItems()) {
          if (checkCanceled(globalTime)) {
            return;
          }
          groupCalculator.calculate(group, global, allUsersGroup);
        }
        finishPortion("Finish recalculating usergroup(s) permissions in: %fs", localTime);

        if (checkCanceled(globalTime)) {
          return;
        }

        localTime = startPortion("Start recalculating user(s) permissions");
        int processed = 0;
        final Sequence<jetbrains.jetpass.api.authority.User> allUsers = PermissionCache.this.getUserDAO().getAllItems();
        while (true) {
          if (checkCanceled(globalTime)) {
            return;
          }

          double usersPackTime = startPortion("Start recalculating user(s) permissions pack");
          Sequence<jetbrains.jetpass.api.authority.User> pack = allUsers.skip(processed).take(PermissionCache.this.getUsersPackSize());
          int counter = 0;

          for (jetbrains.jetpass.api.authority.User user : pack) {
            counter++;
            userCaclulator.calculate(user, global, allUsersGroup);
          }

          processed += counter;
          finishPortion("Finish recalculating user(s) permissions: " + processed + " in: %fs", usersPackTime);

          if (counter < PermissionCache.this.getUsersPackSize()) {
            break;
          }
        }

        finishPortion("Finish recalculating user(s) permissions in: %fs total users processed: " + processed, localTime);

        Map<String, PermissionData> userGroupData = groupCalculator.getData();
        Map<Handler, PermissionData> userData = userCaclulator.getData();
        data = new PermissionCache.Data(userData, userGroupData, serviceCalculator.getData());

        finish("Finish (re)calculating permissions cache in: %fs", globalTime);
        dirty = false;
        canceled = true;
        printStatistics(userData);

        fireCacheCalculated();
      }
    });
  }

  public void setDirty() {
    dirty = true;
  }

  public void cancel() {
    canceled = true;
  }

  public boolean isSuspended() {
    return suspended;
  }

  public void suspend() {
    suspended = true;
  }

  public void resume() {
    suspended = false;
  }

  private boolean checkCanceled(double time) {
    boolean s = canceled;
    if (s) {
      finish("Interrupt permissions cache calculation after: %fs", time);
    }
    return s;
  }

  private void runPrivileged(Runnable privilegedCode) {
    if (this.getPrivilegedRunner() != null) {
      this.getPrivilegedRunner().run(privilegedCode);
    } else {
      privilegedCode.run();
    }
  }

  protected void fireCacheCalculated() {
    if (PermissionCache.this.getListeners() != null && !(PermissionCache.this.getListeners().isEmpty())) {
      for (PermissionCacheListener listener : PermissionCache.this.getListeners()) {
        listener.cacheCalculated(this);
      }
    }
  }

  public UserDAO getUserDAO() {
    return this.myUserDAO;
  }

  public void setUserDAO(UserDAO value) {
    this.myUserDAO = value;
  }

  public UserGroupDAO getGroupDAO() {
    return this.myGroupDAO;
  }

  public void setGroupDAO(UserGroupDAO value) {
    this.myGroupDAO = value;
  }

  public ServiceDAO getServiceDAO() {
    return this.myServiceDAO;
  }

  public void setServiceDAO(ServiceDAO value) {
    this.myServiceDAO = value;
  }

  public ProjectDAO getProjectDAO() {
    return this.myProjectDAO;
  }

  public void setProjectDAO(ProjectDAO value) {
    this.myProjectDAO = value;
  }

  public ResourceDAO getResourceDAO() {
    return this.myResourceDAO;
  }

  public void setResourceDAO(ResourceDAO value) {
    this.myResourceDAO = value;
  }

  public PermissionDAO getPermissionDAO() {
    return this.myPermissionDAO;
  }

  public void setPermissionDAO(PermissionDAO value) {
    this.myPermissionDAO = value;
  }

  public PrivilegedRunner getPrivilegedRunner() {
    return this.myPrivilegedRunner;
  }

  public void setPrivilegedRunner(PrivilegedRunner value) {
    this.myPrivilegedRunner = value;
  }

  public List<PermissionCacheListener> getListeners() {
    return this.myListeners;
  }

  public void setListeners(List<PermissionCacheListener> value) {
    this.myListeners = value;
  }

  public int getUsersPackSize() {
    return this.myUsersPackSize;
  }

  public void setUsersPackSize(int value) {
    this.myUsersPackSize = value;
  }

  private static double startPortion(String message) {
    if (log.isDebugEnabled()) {
      log.debug(message);
    }
    return System.nanoTime();
  }

  private static double start(String message) {
    if (log.isInfoEnabled()) {
      log.info(message);
    }
    return System.nanoTime();
  }

  private static void finishPortion(String message, double time) {
    if (log.isDebugEnabled()) {
      log.debug(String.format(message, (System.nanoTime() - time) / 1000000000));
    }
  }

  private static void finish(String message, double time) {
    if (log.isInfoEnabled()) {
      log.info(String.format(message, (System.nanoTime() - time) / 1000000000));
    }
  }

  private static void printStatistics(Map<Handler, PermissionData> userPermissions) {
    if (log.isInfoEnabled()) {
      log.info(String.format("Permissions cache stats: total user clusters[%d]", userPermissions.keySet().size()));
    }
  }

  private static <T extends IdItem> T toPrincipal(final DAO<T> dao, final String principalUUID) throws IllegalStateException {
    T principal = dao.get(principalUUID);
    if (principal == null) {
      throw new IllegalStateException("Unknown principal " + principalUUID);
    }
    return principal;
  }

  public static boolean isEmpty_dn6lih_a0a0a9(String str) {
    return str == null || str.length() == 0;
  }

  public static boolean isEmpty_dn6lih_a0a0a01(String str) {
    return str == null || str.length() == 0;
  }

  public static boolean isEmpty_dn6lih_a0a0a11(String str) {
    return str == null || str.length() == 0;
  }

  public static interface DataFinder<Key> {
    public PermissionData find();
    public static class StringKey implements PermissionCache.DataFinder<String> {
      private final Map<String, PermissionData> cache;
      private final String principalId;

      public StringKey(Map<String, PermissionData> cache, String principalId) {
        this.cache = cache;
        this.principalId = principalId;
      }

      public PermissionData find() {
        return cache.get(principalId);
      }
    }

    public static class HandlerKey implements PermissionCache.DataFinder<Handler> {
      private final PermissionCache cache;
      private final String principalId;
      private final StringInterner interner;

      public HandlerKey(final PermissionCache cache, final String principalId, final StringInterner interner) {
        this.cache = cache;
        this.principalId = principalId;
        this.interner = interner;
      }

      public PermissionData find() {
        final Handler[] handler = new Handler[1];
        cache.runPrivileged(new Runnable() {
          public void run() {
            handler[0] = new Handler(toPrincipal(cache.getUserDAO(), principalId), interner);
          }
        });
        return cache.getByHandler(handler[0]);
      }
    }

  }

  private static final class Data {
    private final Map<Handler, PermissionData> userPermissions;
    private final Map<String, PermissionData> groupPermissions;
    private final Map<String, PermissionData> servicePermissions;

    private Data(Map<Handler, PermissionData> userPermissions, Map<String, PermissionData> groupPermissions, Map<String, PermissionData> servicePermissions) {
      this.userPermissions = userPermissions;
      this.groupPermissions = groupPermissions;
      this.servicePermissions = servicePermissions;
    }

    private Data() {
      this(new HashMap<Handler, PermissionData>(0), new HashMap<String, PermissionData>(0), new HashMap<String, PermissionData>(0));
    }
  }

  public static class ProjectIDComparator implements Comparator<String> {
    private Map<String, String> id2name = new HashMap<String, String>();
    private final StringInterner interner;
    private String globalUUID;

    public ProjectIDComparator(final StringInterner interner) {
      this.interner = interner;
    }

    public String addProject(Project project) {
      String id = interner.intern(project.getId());
      if (id2name.get(id) != null) {
        return id;
      }
      id2name.put(id, interner.intern(project.getName().toLowerCase()));
      return id;
    }

    public String addGlobal(Project project) {
      // Only first wins! 
      if (globalUUID != null) {
        return globalUUID;
      }
      globalUUID = addProject(project);
      return globalUUID;
    }

    public int compare(String id1, String id2) {
      String name1 = id2name.get(id1);
      String name2 = id2name.get(id2);
      if (name1 == name2) {
        return 0;
      }
      if (name1 == null) {
        return -1;
      }
      if (name2 == null) {
        return 1;
      }
      return name1.compareTo(name2);
    }
  }
}
