package jetbrains.jetpass.dao.api.permissionCache;

/*Generated by MPS */

import jetbrains.jetpass.api.authority.AuthorityHolder;
import java.util.Map;
import java.util.HashMap;
import jetbrains.jetpass.api.security.Project;
import java.util.Set;
import jetbrains.jetpass.api.security.ProjectRole;
import org.jetbrains.annotations.Nullable;
import java.util.HashSet;
import java.util.Iterator;

public abstract class PermissionCalculator<A extends AuthorityHolder, Key, Node extends IPermissionData> {
  private Map<Key, Node> data = new HashMap<Key, Node>();
  protected final IPermissionData.Factory<Node> factory;
  protected final StringInterner interner;

  private PermissionCalculator(final StringInterner interner, IPermissionData.Factory<Node> factory) {
    this.factory = factory;
    this.interner = interner;
  }

  public Node calculate(A principal, Project global, jetbrains.jetpass.api.authority.UserGroup allUsers) {
    Key key = getKey(principal);
    Node node = data.get(key);
    if (node == null) {
      node = factory.create(principal, global);

      Set<Node> parents = getParents(principal, global, allUsers);
      if (parents != null) {
        for (Node parent : parents) {
          node.addParentData(parent);
        }
      }

      for (ProjectRole projectRole : principal.getProjectRoles()) {
        node.addProjectRole(projectRole, interner);
      }

      data.put(key, node);
    }
    return node;
  }

  protected abstract Key getKey(A principal);

  /**
   * Return null if can't have parents to avoid allocation of empty sets
   * 
   * @param principal principal
   * @param global global project
   * @return set of parents or null if parents are impossible
   */
  @Nullable
  protected abstract Set<Node> getParents(A principal, Project global, jetbrains.jetpass.api.authority.UserGroup allUsersGroup);

  public Map<Key, Node> getData() {
    return data;
  }

  public static final class Service<Node extends IPermissionData> extends PermissionCalculator<jetbrains.jetpass.api.Service, String, Node> {
    public Service(StringInterner interner, IPermissionData.Factory<Node> factory) {
      super(interner, factory);
    }

    protected String getKey(jetbrains.jetpass.api.Service principal) {
      return interner.intern(principal.getId());
    }

    protected final Set<Node> getParents(jetbrains.jetpass.api.Service principal, Project global, jetbrains.jetpass.api.authority.UserGroup allUsersGroup) {
      return null;
    }
  }

  public static class UserGroup<Node extends IPermissionData> extends PermissionCalculator<jetbrains.jetpass.api.authority.UserGroup, String, Node> {
    public UserGroup(StringInterner interner, IPermissionData.Factory<Node> factory) {
      super(interner, factory);
    }

    protected String getKey(jetbrains.jetpass.api.authority.UserGroup principal) {
      return interner.intern(principal.getId());
    }

    protected Set<Node> getParents(jetbrains.jetpass.api.authority.UserGroup principal, Project global, jetbrains.jetpass.api.authority.UserGroup allUsersGroup) {
      jetbrains.jetpass.api.authority.UserGroup parent = principal.getParent();

      if (parent == null) {
        return null;
      }

      HashSet<Node> set = new HashSet<Node>(1);
      set.add(calculate(parent, global, allUsersGroup));
      return set;
    }
  }

  public static class User<Node extends IPermissionData> extends PermissionCalculator<jetbrains.jetpass.api.authority.User, Handler, Node> {
    private PermissionCalculator.UserGroup<Node> parent;

    public User(PermissionCalculator.UserGroup<Node> parent) {
      super(parent.interner, parent.factory);
      this.parent = parent;
    }

    public Handler getKey(jetbrains.jetpass.api.authority.User principal) {
      Iterable<? extends ProjectRole> userProjectRoles = principal.getProjectRoles();
      Iterable<? extends jetbrains.jetpass.api.authority.UserGroup> groups = principal.getGroups();
      return new Handler(groups, userProjectRoles, interner);
    }

    @Nullable
    public Set<Node> getParents(jetbrains.jetpass.api.authority.User principal, Project global, jetbrains.jetpass.api.authority.UserGroup allUsersGroup) {
      HashSet<Node> parents = new HashSet<Node>();

      Iterator<? extends jetbrains.jetpass.api.authority.UserGroup> groups = principal.getGroups().iterator();
      if (groups.hasNext()) {
        while (groups.hasNext()) {
          jetbrains.jetpass.api.authority.UserGroup group = groups.next();
          parents.add(parent.calculate(group, global, allUsersGroup));
        }
      } else {
        parents.add(parent.calculate(allUsersGroup, global, allUsersGroup));
      }
      return parents;
    }
  }
}
