package jetbrains.jetpass.dao.api.permissionCache;

/*Generated by MPS */

import java.util.Set;
import java.util.Collections;
import java.util.Map;
import java.util.HashMap;
import java.util.HashSet;
import jetbrains.jetpass.api.security.Project;
import jetbrains.jetpass.api.security.ProjectRole;
import jetbrains.jetpass.api.security.Role;
import jetbrains.jetpass.api.security.Permission;
import jetbrains.jetpass.api.authority.AuthorityHolder;

public class PermissionData implements IPermissionData<PermissionData> {
  public static final PermissionData EMPTY = new PermissionData(((String) null), new PermissionCache.ProjectIDComparator(new StringInterner()));
  protected static final Set<String> EMPTY_SET = Collections.emptySet();

  private final Map<String, Set<String>> role2project = new HashMap<String, Set<String>>();
  private final Map<String, Set<String>> permission2project = new HashMap<String, Set<String>>();
  private final Map<String, Set<String>> project2role = new HashMap<String, Set<String>>();
  private final Set<String> globalPermissions = new HashSet<String>();
  private final String globalProjectUUID;
  private final PermissionCache.ProjectIDComparator comparator;

  private PermissionData(String globalProjectUUID, PermissionCache.ProjectIDComparator comparator) {
    this.globalProjectUUID = globalProjectUUID;
    this.comparator = comparator;
  }

  public PermissionData(Project global, PermissionCache.ProjectIDComparator projectIDComparator) {
    this(projectIDComparator.addGlobal(global), projectIDComparator);
  }

  public void addParentData(PermissionData permissionData) {
    deepCopy(role2project, permissionData.role2project);
    deepCopy(permission2project, permissionData.permission2project);
    deepCopy(project2role, permissionData.project2role);
    globalPermissions.addAll(permissionData.globalPermissions);
  }

  public void addProjectRole(ProjectRole projectRole, StringInterner interner) {
    Role role = projectRole.getRole();
    Project project = projectRole.getProject();

    comparator.addProject(project);

    String projectUUID = interner.intern(project.getId());
    String roleUUID = interner.intern(role.getId());
    addRole2ProjectAndProject2Role(projectUUID, roleUUID);


    // Add permission2project 
    Iterable<? extends Permission> permissions = role.getPermissions();
    for (Permission p : permissions) {
      addPermission2Project(projectUUID, interner.intern(p.getId()), eq_fc0t24_a0c0a0m0b(globalProjectUUID, projectUUID) || p.isGlobal());
    }

  }

  private void addRole2ProjectAndProject2Role(String projectUUID, String roleUUID) {
    // Add role2project 
    Set<String> projects = role2project.get(roleUUID);
    if (projects == null) {
      projects = new HashSet<String>();
      role2project.put(roleUUID, projects);
    }
    projects.add(projectUUID);

    // Add project2role 
    Set<String> roles = project2role.get(projectUUID);
    if (roles == null) {
      roles = new HashSet<String>();
      project2role.put(projectUUID, roles);
    }
    roles.add(roleUUID);
  }

  private void addPermission2Project(String projectUUID, String permissionUUID, boolean global) {
    Set<String> permissionProjects = permission2project.get(permissionUUID);
    if (permissionProjects == null) {
      permissionProjects = new HashSet<String>();
      permission2project.put(permissionUUID, permissionProjects);
    }
    permissionProjects.add(projectUUID);

    if (global) {
      globalPermissions.add(permissionUUID);
    }
  }

  public Set<String> getRole2Project(String id) {
    Set<String> result = role2project.get(id);
    return (result == null ?
      EMPTY_SET :
      result
    );
  }

  public Set<String> getRole2ProjectKeySet() {
    return role2project.keySet();
  }

  public Set<String> getPermission2project(String id) {
    Set<String> result = permission2project.get(id);
    return (result == null ?
      EMPTY_SET :
      result
    );
  }

  public Set<String> getPermission2projectKeySet() {
    return permission2project.keySet();
  }

  public Set<String> getProject2role(String id) {
    Set<String> result = project2role.get(id);
    return (result == null ?
      EMPTY_SET :
      result
    );
  }

  public Set<String> getProject2roleKeySet() {
    return project2role.keySet();
  }

  public boolean isProject2roleEmpty() {
    return project2role.isEmpty();
  }

  public Set<String> getGlobalRoles() {
    Set<String> result = project2role.get(globalProjectUUID);
    return (result == null ?
      EMPTY_SET :
      result
    );
  }

  public Set<String> getGlobalPermissions() {
    return globalPermissions;
  }

  public PermissionQuery toQuery() {
    return new PermissionQuery(this, globalProjectUUID, comparator);
  }

  public String getGlobalProjectUUID() {
    return globalProjectUUID;
  }

  private static void deepCopy(final Map<String, Set<String>> target, final Map<String, Set<String>> source) {
    if (source.size() == 0) {
      return;
    }

    for (Map.Entry<String, Set<String>> entry : source.entrySet()) {
      Set<String> set = target.get(entry.getKey());
      if (set == null) {
        set = new HashSet<String>(entry.getValue().size());
        target.put(entry.getKey(), set);
      }
      set.addAll(entry.getValue());
    }
  }

  public static PermissionData.Factory createFactory(StringInterner interner) {
    return new PermissionData.Factory(new PermissionCache.ProjectIDComparator(interner));
  }

  private static boolean eq_fc0t24_a0c0a0m0b(Object a, Object b) {
    return (a != null ?
      a.equals(b) :
      a == b
    );
  }

  public static class Factory extends IPermissionData.Factory<PermissionData> {
    private final PermissionCache.ProjectIDComparator comparator;

    protected Factory(PermissionCache.ProjectIDComparator comparator) {
      this.comparator = comparator;
    }

    public PermissionData create(AuthorityHolder principal, Project global) {
      return new PermissionData(global, comparator);
    }
  }
}
