package jetbrains.jetpass.dao.api.permissionCache;

/*Generated by MPS */

import java.util.Comparator;
import java.util.Set;
import java.util.TreeSet;
import org.jetbrains.annotations.Nullable;

public class PermissionQuery extends BasePermissionQuery<PermissionQuery> {
  private PermissionData permissionData;
  private Comparator<String> projectIDComparator;

  public PermissionQuery(PermissionData permissionData, String globalProjectUUID, Comparator<String> projectIDComparator) {
    super(globalProjectUUID);
    this.permissionData = permissionData;
    this.projectIDComparator = projectIDComparator;
  }

  @Deprecated
  public PermissionQuery toSpace(String spaceUUID) {
    return toProject(spaceUUID);
  }

  @Deprecated
  public PermissionQuery toGlobalSpace() {
    return toGlobalProject();
  }

  public boolean canAccess() {
    if (canAccessGlobal()) {
      return true;
    }
    if (rolesUUID != null) {
      if (projectUUID != null) {
        return permissionData.getRole2Project(rolesUUID).contains(projectUUID);
      }
      return permissionData.getRole2Project(rolesUUID) != PermissionData.EMPTY;
    } else if (permissionUUID != null) {
      if (projectUUID != null) {
        return permissionData.getPermission2project(permissionUUID).contains(projectUUID);
      }
      return permissionData.getPermission2project(permissionUUID) != PermissionData.EMPTY_SET;
    } else {
      if (projectUUID != null) {
        return permissionData.getProject2role(projectUUID) != PermissionData.EMPTY_SET;
      }
      return !(permissionData.isProject2roleEmpty());
    }
  }

  public boolean canAccessGlobal() {
    boolean result;
    if (rolesUUID != null) {
      result = permissionData.getGlobalRoles().contains(rolesUUID);
    } else if (permissionUUID != null) {
      result = permissionData.getGlobalPermissions().contains(permissionUUID);
    } else {
      result = !(permissionData.getGlobalPermissions().isEmpty());
    }
    return result;
  }

  @Deprecated
  public Set<String> getAccessibleSpaces() {
    return getAccessibleProjects();
  }

  public Set<String> getAccessibleProjects() {
    return getAccessibleProjects(projectIDComparator);
  }

  public Set<String> getAccessibleProjects(Comparator<String> comparator) {
    Set<String> projectUUIDs = new TreeSet<String>(comparator);
    if (rolesUUID != null) {
      projectUUIDs.addAll(permissionData.getRole2Project(rolesUUID));
    } else if (permissionUUID != null) {
      projectUUIDs.addAll(permissionData.getPermission2project(permissionUUID));
    } else {
      projectUUIDs.addAll(permissionData.getProject2roleKeySet());
    }
    return projectUUIDs;
  }

  public Set<String> getGrantedRoles() {
    return getGrantedRoles(null);
  }

  public Set<String> getGrantedRoles(@Nullable Comparator<String> comparator) {
    TreeSet<String> roleUUIDs = new TreeSet<String>(comparator);
    if (projectUUID != null) {
      roleUUIDs.addAll(permissionData.getGlobalRoles());
      roleUUIDs.addAll(permissionData.getProject2role(projectUUID));
    } else {
      roleUUIDs.addAll(permissionData.getRole2ProjectKeySet());
    }
    return roleUUIDs;
  }

  public Set<String> getGrantedPermissions() {
    return getGrantedPermissions(null);
  }

  public Set<String> getGrantedPermissions(@Nullable Comparator<String> comparator) {
    TreeSet<String> permissions = new TreeSet<String>(comparator);
    if (projectUUID != null) {
      permissions.addAll(permissionData.getGlobalPermissions());
      for (String permission : permissionData.getPermission2projectKeySet()) {
        if (permissionData.getPermission2project(permission).contains(projectUUID)) {
          permissions.add(permission);
        }
      }
    } else {
      permissions.addAll(permissionData.getPermission2projectKeySet());
    }
    return permissions;
  }

  @Override
  public PermissionQuery clone() {
    return new PermissionQuery(permissionData, globalProjectUUID, projectIDComparator).copy(this);
  }

  public boolean contains(PermissionQuery that) {
    Set<String> thisGlobalPermissions = this.permissionData.getGlobalPermissions();
    for (String permissionUUID : that.permissionData.getPermission2projectKeySet()) {
      if (thisGlobalPermissions.contains(permissionUUID)) {
        continue;
      }
      Set<String> thisProjects = permissionData.getPermission2project(permissionUUID);
      if (thisProjects == null || thisProjects.isEmpty()) {
        return false;
      }
      for (String projectUUID : that.permissionData.getPermission2project(permissionUUID)) {
        if (!(thisProjects.contains(projectUUID))) {
          return false;
        }
      }
    }
    return true;
  }
}
