package jetbrains.jetpass.dao.api.permissionCache;

/*Generated by MPS */

import java.util.Set;
import java.util.Collections;
import jetbrains.jetpass.dao.api.ServiceDAO;
import jetbrains.jetpass.dao.api.security.ProjectDAO;
import jetbrains.jetpass.dao.api.security.PermissionDAO;
import jetbrains.jetpass.sequence.Sequence;
import jetbrains.jetpass.api.NamedItem;
import java.util.TreeMap;
import java.util.LinkedHashSet;

public class ServicePermissionQuery extends PermissionQuery {
  private static final Set<String> EMPTY = Collections.emptySet();

  private final boolean trusted;
  private final Set<String> allProjects;
  private Set<String> allPermissions;

  private ServicePermissionQuery(String globalProjectUUID, boolean trusted, Set<String> allProjects, Set<String> allPermissions) {
    super(null, globalProjectUUID, null);
    this.trusted = trusted;
    this.allProjects = allProjects;
    this.allPermissions = allPermissions;
  }

  @Override
  public boolean canAccess() {
    return trusted;
  }

  @Override
  public boolean canAccessGlobal() {
    return trusted;
  }

  @Override
  public Set<String> getAccessibleProjects() {
    return (trusted ?
      allProjects :
      EMPTY
    );
  }

  @Override
  public Set<String> getGrantedRoles() {
    return EMPTY;
  }

  @Override
  public Set<String> getGrantedPermissions() {
    return (trusted ?
      allPermissions :
      EMPTY
    );
  }

  @Override
  public PermissionQuery clone() {
    return new ServicePermissionQuery(globalProjectUUID, trusted, allProjects, allPermissions).copy(this);
  }

  @Override
  public boolean contains(PermissionQuery that) {
    return trusted;
  }

  public static ServicePermissionQuery create(final ServiceDAO serviceDAO, final String serviceID, final ProjectDAO projectDAO, final PermissionDAO permissionDAO) {
    String globalProjectID = projectDAO.getGlobal().getId();
    boolean trusted = serviceDAO.get(serviceID).isTrusted();
    Set<String> allProjects = toIds(projectDAO.getAllItems());
    Set<String> allPermissions = toIds(permissionDAO.getAllItems());
    return new ServicePermissionQuery(globalProjectID, trusted, allProjects, allPermissions);
  }

  private static Set<String> toIds(Sequence<? extends NamedItem> sequence) {
    TreeMap<String, String> result = new TreeMap<String, String>();
    for (NamedItem item : sequence) {
      result.put(item.getName(), item.getId());
    }
    return new LinkedHashSet<String>(result.values());
  }
}
