package jetbrains.jetpass.dao.remote;

/*Generated by MPS */

import jetbrains.jetpass.dao.api.API;
import org.jetbrains.annotations.NotNull;
import jetbrains.jetpass.dao.remote.api.RemoteAPI;
import jetbrains.jetpass.dao.api.event.EventDAO;
import jetbrains.jetpass.dao.api.ServiceDAO;
import jetbrains.jetpass.dao.api.security.ResourceDAO;
import jetbrains.jetpass.dao.api.security.SpaceRoleDAO;
import jetbrains.jetpass.dao.api.security.ProjectRoleDAO;
import jetbrains.jetpass.dao.api.security.PermissionDAO;
import jetbrains.jetpass.dao.api.security.RoleDAO;
import jetbrains.jetpass.dao.api.authority.UserDAO;
import jetbrains.jetpass.dao.api.security.SpaceDAO;
import jetbrains.jetpass.dao.api.security.ProjectDAO;
import jetbrains.jetpass.dao.api.authority.UserGroupDAO;
import jetbrains.jetpass.dao.api.authority.UserDetailsDAO;
import jetbrains.jetpass.dao.api.settings.SettingsDAO;
import jetbrains.jetpass.dao.api.authority.AuthModuleDAO;
import jetbrains.jetpass.dao.api.ssl.KeyStoreDAO;
import jetbrains.jetpass.dao.api.ssl.CertificateDAO;
import jetbrains.jetpass.dao.api.metrics.MetricsDAO;
import jetbrains.jetpass.client.hub.HubClient;
import jetbrains.jetpass.client.hub.ClientBuilder;
import jetbrains.jetpass.dao.remote.api.ScheduledExecutor;
import org.jetbrains.annotations.Nullable;
import java.net.URL;
import java.net.URI;
import java.util.concurrent.ScheduledExecutorService;
import jetbrains.jetpass.api.Service;

public class HubAPI implements API {
  public static final long DEFAULT_SYNC_INTEVAL = 15 * 1000;
  private static final long MIN_SYNC_INTERVAL = 1000;

  @NotNull
  private final RemoteAPI api;
  @NotNull
  private final TokenService tokenService;

  private HubAPI(@NotNull final RemoteAPI api, HubAPI.HubAPIBuilder builder) {
    this.api = api;
    tokenService = new TokenService(builder.treatEmptyTokenAsGuest, api);
  }

  @NotNull
  public EventDAO getEventDAO() {
    return api.getEventDAO();
  }

  @NotNull
  public ServiceDAO getServiceDAO() {
    return api.getServiceDAO();
  }

  @NotNull
  public ResourceDAO getResourceDAO() {
    return api.getResourceDAO();
  }

  @NotNull
  public SpaceRoleDAO getSpaceRoleDAO() {
    return api.getSpaceRoleDAO();
  }

  @NotNull
  public ProjectRoleDAO getProjectRoleDAO() {
    return api.getProjectRoleDAO();
  }

  @NotNull
  public PermissionDAO getPermissionDAO() {
    return api.getPermissionDAO();
  }

  @NotNull
  public RoleDAO getRoleDAO() {
    return api.getRoleDAO();
  }

  @NotNull
  public UserDAO getUserDAO() {
    return api.getUserDAO();
  }

  @NotNull
  public SpaceDAO getSpaceDAO() {
    return api.getSpaceDAO();
  }

  @NotNull
  public ProjectDAO getProjectDAO() {
    return api.getProjectDAO();
  }

  @NotNull
  public UserGroupDAO getUserGroupDAO() {
    return api.getUserGroupDAO();
  }

  @NotNull
  public UserDetailsDAO getUserDetailsDAO() {
    return api.getUserDetailsDAO();
  }

  @NotNull
  public SettingsDAO getSettingsDAO() {
    return api.getSettingsDAO();
  }

  @NotNull
  public AuthModuleDAO getAuthModuleDAO() {
    return api.getAuthModuleDAO();
  }

  @NotNull
  public KeyStoreDAO getKeyStoreDAO() {
    return api.getKeyStoreDAO();
  }

  @NotNull
  public CertificateDAO getCertificateDAO() {
    return api.getCertificateDAO();
  }

  @NotNull
  public MetricsDAO getMetricsDAO() {
    // TODO! 
    return null;
  }

  @NotNull
  public TokenService getTokenService() {
    return tokenService;
  }

  public void syncNow() {
    api.syncNow();
  }

  public void syncCheck() {
    api.syncCheck();
  }

  public void resyncAll() {
    api.resyncAll();
  }

  public void resetLastSync() {
    api.outOfSync();
  }

  @NotNull
  public static HubAPI.HubAPIBuilder builder() {
    return builder(HubClient.builder());
  }

  @NotNull
  public static HubAPI.HubAPIBuilder builder(HubClient.HubClientBuilder clientBuilder) {
    return new HubAPI.HubAPIBuilder(clientBuilder);
  }

  public static class HubAPIBuilder implements ClientBuilder<HubAPI> {
    private final HubClient.HubClientBuilder hubClientBuilder;
    private long syncInterval;
    private ScheduledExecutor executor;
    private String serviceID;
    private String serviceSecret;
    private boolean treatEmptyTokenAsGuest = true;

    private HubAPIBuilder(HubClient.HubClientBuilder hubClientBuilder) {
      this.hubClientBuilder = hubClientBuilder;
      syncInterval(DEFAULT_SYNC_INTEVAL);
    }

    @NotNull
    public HubAPI.HubAPIBuilder baseUrl(@Nullable String baseUrl) {
      hubClientBuilder.baseUrl(baseUrl);
      return this;
    }

    @NotNull
    public HubAPI.HubAPIBuilder baseUrl(@Nullable URL baseUrl) {
      hubClientBuilder.baseUrl(baseUrl);
      return this;
    }

    @NotNull
    public HubAPI.HubAPIBuilder baseUrl(@Nullable URI baseUri) {
      hubClientBuilder.baseUrl(baseUri);
      return this;
    }

    @NotNull
    public ClientBuilder<HubAPI> readTimeout(@Nullable Integer timeout) {
      hubClientBuilder.readTimeout(timeout);
      return this;
    }

    @NotNull
    public ClientBuilder<HubAPI> connectTimeout(@Nullable Integer timeout) {
      hubClientBuilder.connectTimeout(timeout);
      return this;
    }

    /**
     * Sync interval in millis (invalidation is made on get request)
     * Default sync interval 15 000 ms
     * 
     * @param millis positive interval (1000 is minimum)
     */
    @NotNull
    public HubAPI.HubAPIBuilder syncInterval(long millis) {
      syncInterval = (millis < MIN_SYNC_INTERVAL ?
        MIN_SYNC_INTERVAL :
        millis
      );
      return this;
    }

    /**
     * Executor for periodical synchornization check
     * 
     * @param executor which executes update periodically
     * @return HupAPIBuilder
     */
    @NotNull
    public HubAPI.HubAPIBuilder executor(@Nullable ScheduledExecutor executor) {
      this.executor = executor;
      return this;
    }

    /**
     * Executor for periodical synchornization check
     * 
     * @param executor which executes update periodically
     * @return HupAPIBuilder
     */
    @NotNull
    public HubAPI.HubAPIBuilder executor(@Nullable ScheduledExecutorService executor) {
      return (executor == null ?
        this :
        executor(new ScheduledExecutorAdapter(executor))
      );
    }

    @NotNull
    public HubAPI.HubAPIBuilder service(@Nullable Service service) {
      return (service == null ?
        this :
        service(service.getId(), service.getSecret())
      );
    }

    @NotNull
    public HubAPI.HubAPIBuilder service(@Nullable String serviceID, @Nullable String serviceSecret) {
      this.serviceID = serviceID;
      this.serviceSecret = serviceSecret;
      return this;
    }

    @NotNull
    public HubAPI.HubAPIBuilder treatEmptyTokenAsGuestUser(boolean treat) {
      this.treatEmptyTokenAsGuest = treat;
      return this;
    }

    public HubAPI build() {
      RemoteAPI remoteAPI = RemoteAPI.create(hubClientBuilder.build().getAccountsClient(serviceID, serviceSecret), syncInterval, executor);
      return new HubAPI(remoteAPI, this);
    }
  }
}
