package jetbrains.jetpass.dao.remote;

/*Generated by MPS */

import jetbrains.jetpass.dao.remote.api.RemoteAPI;
import org.jetbrains.annotations.NotNull;
import org.jetbrains.annotations.Nullable;
import com.intellij.hub.auth.oauth2.token.AccessToken;
import com.intellij.hub.auth.InvalidTokenException;
import jetbrains.jetpass.dao.api.authority.AuthorityHolderDAO;
import jetbrains.jetpass.api.security.Resource;

public class TokenService {
  private final boolean treatEmptyTokenAsGuest;
  private final RemoteAPI api;

  public TokenService(boolean treatEmptyTokenAsGuest, @NotNull RemoteAPI api) {
    this.treatEmptyTokenAsGuest = treatEmptyTokenAsGuest;
    this.api = api;
  }

  @Nullable
  public AccessToken checkToken(@Nullable String header) throws InvalidTokenException {
    return api.getClient().getOAuthRequestValidator().getAccessToken(header);
  }

  public boolean hasPermission(@Nullable String token, String permissionId, String resourceOrProjectId) {
    String userId = null;
    String serviceId = null;
    if ((token == null || token.length() == 0) && treatEmptyTokenAsGuest) {
      userId = api.getUserDAO().getGuest().getId();
    } else {
      try {
        AccessToken t = checkToken(token);
        userId = t.getUser();
        serviceId = t.getClient();
      } catch (InvalidTokenException ex) {
        return false;
      }
    }
    return buildQuery(userId, serviceId).hasPermission(permissionId, resourceOrProjectId);
  }

  @NotNull
  private TokenService.PermissionQuery buildQuery(@Nullable String userId, @Nullable String serviceId) {
    if (userId != null) {
      return new TokenService.DAOPermissionQuery(userId, api.getUserDAO());
    } else if (serviceId != null) {
      return new TokenService.DAOPermissionQuery(serviceId, api.getServiceDAO());
    } else {
      return new TokenService.FalsePermissionQuery();
    }
  }

  private static interface PermissionQuery {
    public boolean hasPermission(String permissionId, String resourceOrProjectId);
  }

  private class DAOPermissionQuery implements TokenService.PermissionQuery {
    private final String authHolderId;
    private final AuthorityHolderDAO dao;

    private DAOPermissionQuery(String authHolderId, AuthorityHolderDAO dao) {
      this.authHolderId = authHolderId;
      this.dao = dao;
    }

    public boolean hasPermission(String permissionId, String resourceOrProjectId) {
      Resource resource = api.getResourceDAO().get(resourceOrProjectId);
      if (resource != null) {
        return dao.hasPermission(authHolderId, api.getPermissionDAO().get(permissionId), resource);
      } else {
        return dao.hasPermission(authHolderId, api.getPermissionDAO().get(permissionId), api.getProjectDAO().get(resourceOrProjectId));
      }
    }
  }

  private class FalsePermissionQuery implements TokenService.PermissionQuery {
    private FalsePermissionQuery() {
    }

    public boolean hasPermission(String permissionId, String resourceOrProjectId) {
      return false;
    }
  }
}
