package jetbrains.jetpass.dao.remote.api;

/*Generated by MPS */

import jetbrains.jetpass.api.IdItem;
import jetbrains.jetpass.client.BaseField;
import jetbrains.jetpass.dao.api.DAO;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import jetbrains.jetpass.client.FieldPartial;
import java.util.concurrent.atomic.AtomicLong;
import org.jetbrains.annotations.Nullable;
import org.jetbrains.annotations.NotNull;
import jetbrains.jetpass.dao.api.ItemNotFoundException;
import jetbrains.jetpass.sequence.Sequence;
import jetbrains.jetpass.sequence.impl.TransformFunction;
import java.util.Iterator;
import jetbrains.jetpass.api.event.Event;
import jetbrains.jetpass.client.BaseFilter;
import jetbrains.mps.webr.rpc.rest.json.common.runtime.Page;
import jetbrains.jetpass.dao.api.DataAccessException;
import javax.ws.rs.WebApplicationException;
import javax.ws.rs.core.Response;
import jetbrains.jetpass.rest.dto.ErrorJSON;
import jetbrains.jetpass.dao.api.DataWriteException;
import jetbrains.jetpass.dao.api.NotUniqueFieldException;
import jetbrains.jetpass.dao.api.SecurityViolationException;
import jetbrains.jetpass.dao.api.ServiceIsReadonlyException;

public abstract class RemoteDAO<I extends IdItem, J extends I, F extends BaseField> implements RemoteItemStore<I, J>, DAO<I> {
  private static final long NEVER_SYNCED = -1;
  protected static Log log = LogFactory.getLog(RemoteDAO.class);

  protected FieldPartial<F> minimalPartial;
  protected FieldPartial<F> _partial;
  protected final RemoteDataStore<I, J> store = new RemoteDataStore<I, J>();
  private final AtomicLong lastSync = new AtomicLong(NEVER_SYNCED);
  protected RemoteItemAccessor<I, J> itemAccessor;
  private RemoteAPI myRemoteAPI;

  public RemoteDAO(FieldPartial<F> minimalPartial) {
    this.minimalPartial = minimalPartial;
    this._partial = minimalPartial;
  }

  @Nullable
  public I get(String id) {
    if ((id == null || id.length() == 0)) {
      return null;
    }
    RemoteItemData<I, J> data = getData(id);
    if (data == null) {
      data = put(requestRemoteItem(id, _partial));
    }
    return (data == null ?
      null :
      data.getItem()
    );
  }

  @Nullable
  protected RemoteItemData<I, J> put(@Nullable J item) {
    return putRemoteItem(store, item, _partial);
  }

  @Nullable
  public RemoteItemData<I, J> getData(String id) {
    ConstraintsUtil.checkNotEmptyId(id);
    return store.getData(id);
  }

  @NotNull
  public I getNotNull(String id) throws ItemNotFoundException {
    I item = get(id);
    if (item == null) {
      throw new ItemNotFoundException(itemAccessor.getItemType().getSimpleName(), id);
    }
    return item;
  }

  public Sequence<I> getAllItems() {
    if (isSynced()) {
      return this.store.getAllData().select(new TransformFunction<RemoteItemData<I, J>, I>() {
        public I transform(RemoteItemData<I, J> data) {
          return data.getItem();
        }
      });
    } else {
      return new RemoteSequence<I, J>(((RemoteDAO<I, J, BaseField>) this));
    }
  }

  public long getVersion() {
    return store.getVersion();
  }

  public final void sync() {
    synchronized (lastSync) {
      if (log.isInfoEnabled()) {
        log.info(String.format("Start %s sync", getClass().getSimpleName()));
      }
      long syncTime = System.currentTimeMillis();
      doSync(syncTime);
      lastSync.set(syncTime);
      if (log.isInfoEnabled()) {
        log.info(String.format("Finished %s sync", getClass().getSimpleName()));
      }
    }
  }

  protected void doSync(long syncStartTime) {
    RemoteDataStore<I, J> newStore = new RemoteDataStore<I, J>();

    final FieldPartial<F> partial = this._partial;
    Iterator<J> iter = createIterator(filter(), partial);

    for (J dto; iter.hasNext();) {
      dto = iter.next();

      onNewItem(dto);

      putRemoteItem(newStore, dto, partial);
    }

    store.merge(newStore);
  }

  public final boolean isSynced() {
    return lastSync.get() > NEVER_SYNCED;
  }

  protected void onNewItem(final J dto) {
    // Do nothing 
  }

  @Nullable
  protected RemoteItemData<I, J> initData(@Nullable J dto, final FieldPartial<F> partial) {
    if (dto == null) {
      if (log.isDebugEnabled()) {
        log.debug("Tried to init empty data", new Throwable());
      }
      return null;
    }

    RemoteItemAccessor<I, J> itemAccessor = getItemAccessor(dto);

    if (itemAccessor == null) {
      if (log.isWarnEnabled()) {
        log.warn("Couldn't find ItemAccessor for item: " + dto.getId() + " (very likely client is outdated)");
      }
      return null;
    }

    RemoteItemData<I, J> itemData = itemAccessor.newRemoteItemData();

    for (String fieldName : partial.getFieldNameSet()) {
      itemAccessor.setFieldValue(itemData, fieldName, dto);
    }
    return itemData;
  }

  public void newEvent(Event event) {
    Event.Type eventType = Event.Type.fromString(event.getType());
    String itemId = event.getTargetId();
    switch (eventType) {
      case CREATE:
      case UPDATE:
        put(requestRemoteItem(itemId, _partial));
        break;
      case REMOVE:
        removeRemoteItem(store, itemId);
        break;
      default:
        throw new IllegalArgumentException("Unknown event type: " + eventType.getName());
    }
  }

  protected Iterator<J> createIterator(final BaseFilter filter, final FieldPartial<F> partial) {
    return new PageIterator<J>() {
      protected Page<J> fetch(int skip, int top) {
        return RemoteDAO.this.fetchPage(filter.skip(skip).top(top), partial);
      }
    };
  }

  protected void removeRemoteItem(RemoteDataStore<I, J> store, String itemId) {
    store.removeData(itemId);
  }

  @Nullable
  protected RemoteItemData<I, J> putRemoteItem(RemoteDataStore<I, J> store, @Nullable J data, FieldPartial<F> partial) {
    RemoteItemData<I, J> itemData = null;
    if (data != null) {
      itemData = initData(data, partial);
      if (itemData != null) {
        // Item data can be null if client outdated =( 
        store.putData(data.getId(), itemData);

        Iterable<String> aliasIds = data.getAliasIds();
        if (aliasIds != null) {
          for (String aliasId : aliasIds) {
            store.putData(aliasId, itemData);
          }
        }
      }
    } else {
      if (log.isDebugEnabled()) {
        log.debug("Tried to put empty data", new Throwable());
      }
    }
    return itemData;
  }

  protected abstract FieldPartial<F> createPartial(String fieldName);

  protected abstract Page<J> fetchPage(BaseFilter filter, FieldPartial<F> partial);

  protected abstract J fetchItem(String id, FieldPartial<F> partial);

  protected BaseFilter filter() {
    return new RemoteDAO.DefaultFilter();
  }

  public void requestFieldValue(String itemId, String fieldName) {
    RemoteItemData<I, J> data = store.getData(itemId);
    if (data == null) {
      throw new IllegalStateException("Can't ensure unknown item");
    }
    String fetchInfo = "Fetch item field \"" + fieldName + "\" of item [" + itemId + "]";
    if (log.isInfoEnabled()) {
      log.info(fetchInfo);
    }

    RemoteDAO.this.getRemoteAPI().fireFieldFetchRequested(this, itemId, fieldName);
    J dto = requestRemoteItem(itemId, createPartial(fieldName));

    if (dto == null) {
      throw new IllegalStateException("Item was removed. Failed: " + fetchInfo);
    }

    getItemAccessor(dto).setFieldValue(data, fieldName, dto);
  }

  @Nullable
  private J requestRemoteItem(String id, FieldPartial<F> partial) {
    J dto = fetchItem(id, partial);
    if (dto == null) {
      StringBuilder error = new StringBuilder("Failed to load fields [");
      error.append(partial.toString());
      error.append("] of item [").append(id).append("]");
      if (log.isInfoEnabled()) {
        log.info(error.toString());
      }
    }
    return dto;
  }

  @Nullable
  public RemoteItemAccessor<I, J> getItemAccessor(J dto) {
    return itemAccessor;
  }

  protected final DataAccessException wrapClientException(WebApplicationException ex, String id) {
    Response response = ex.getResponse();
    int status = response.getStatus();
    String message = ex.getMessage();
    ErrorJSON error = null;

    try {
      error = response.readEntity(ErrorJSON.class);
      message = error.getError_description();
    } catch (RuntimeException re) {
      if (log.isWarnEnabled()) {
        log.warn("Failed to decode ErrorJSON", re);
      }
    }

    if (status == Response.Status.NOT_FOUND.getStatusCode()) {
      return new ItemNotFoundException(itemAccessor.getItemType().getSimpleName(), id);
    }
    if (status == Response.Status.BAD_REQUEST.getStatusCode()) {
      String devMessage = null;
      if (error != null) {
        devMessage = error.getError_developer_message();
      }
      return new DataWriteException(((devMessage != null && devMessage.length() > 0) ?
        devMessage :
        message
      ), ex);
    }
    if (status == Response.Status.CONFLICT.getStatusCode()) {
      return new NotUniqueFieldException(itemAccessor.getItemType().getSimpleName(), "id");
    }
    if (status == Response.Status.FORBIDDEN.getStatusCode()) {
      return new SecurityViolationException(message, ex);
    }
    if (status == Response.Status.SERVICE_UNAVAILABLE.getStatusCode() && ServiceIsReadonlyException.READONLY_SERVICE_MESSAGE.equals(message)) {
      return new ServiceIsReadonlyException();
    }
    return new DataAccessException(message, ex);
  }

  public FieldPartial<F> getDefaultPartial() {
    return _partial;
  }

  public void setDefaultPartial(FieldPartial<F> value) {
    _partial = minimalPartial.union(value);
  }

  public RemoteAPI getRemoteAPI() {
    return this.myRemoteAPI;
  }

  public void setRemoteAPI(RemoteAPI value) {
    this.myRemoteAPI = value;
  }

  private static class DefaultFilter extends BaseFilter<BaseFilter> {
    private DefaultFilter() {
    }
  }
}
