package jetbrains.jetpass.dao.remote.api;

/*Generated by MPS */

import jetbrains.jetpass.api.IdItem;
import java.util.concurrent.atomic.AtomicLong;
import java.util.Map;
import java.util.LinkedHashMap;
import java.util.concurrent.locks.ReentrantReadWriteLock;
import java.util.concurrent.locks.Lock;
import jetbrains.jetpass.sequence.Sequence;
import java.util.HashSet;
import jetbrains.jetpass.sequence.impl.TransformSequence;
import jetbrains.jetpass.sequence.impl.IterableSequence;
import jetbrains.jetpass.sequence.impl.TransformFunction;
import java.util.ConcurrentModificationException;

public class RemoteDataStore<I extends IdItem, J extends I> {
  private final AtomicLong version = new AtomicLong(0);
  private final Map<String, RemoteItemData<I, J>> data = new LinkedHashMap<String, RemoteItemData<I, J>>();
  private final ReentrantReadWriteLock rwl = new ReentrantReadWriteLock();
  private final Lock readLock = rwl.readLock();
  private final Lock writeLock = rwl.writeLock();

  public RemoteDataStore() {
  }

  public long getVersion() {
    return version.get();
  }

  public Sequence<RemoteItemData<I, J>> getAllData() {
    try {
      readLock.lock();
      // FIX: concurrent modification exception may occur 
      final long version = this.version.get();
      HashSet<String> copy = new HashSet<String>(data.keySet());
      return new TransformSequence(new IterableSequence(copy), new TransformFunction<String, RemoteItemData<I, J>>() {
        public RemoteItemData<I, J> transform(String id) {
          if (getVersion() > version) {
            // we've got another 
            throw new ConcurrentModificationException();
          }
          return getData(id);
        }
      });
    } finally {
      readLock.unlock();
    }
  }

  public RemoteItemData<I, J> getData(String itemId) {
    try {
      readLock.lock();
      return data.get(itemId);
    } finally {
      readLock.unlock();
    }
  }

  public boolean removeData(String itemId) {
    try {
      writeLock.lock();
      boolean removed = data.remove(itemId) != null;
      if (removed) {
        version.incrementAndGet();
      }
      return removed;
    } finally {
      writeLock.unlock();
    }
  }

  protected void removeAll() {
    try {
      writeLock.lock();
      data.clear();
      version.incrementAndGet();
    } finally {
      writeLock.unlock();
    }
  }

  public boolean putData(String itemId, RemoteItemData<I, J> item) {
    try {
      writeLock.lock();
      boolean added = data.put(itemId, item) != null;
      if (added) {
        version.incrementAndGet();
      }
      return added;
    } finally {
      writeLock.unlock();
    }
  }

  public void merge(RemoteDataStore<I, J> store) {
    try {
      writeLock.lock();
      store.writeLock.lock();
      version.incrementAndGet();
      data.clear();
      data.putAll(store.data);
    } finally {
      store.writeLock.unlock();
      writeLock.unlock();
    }
  }
}
