package jetbrains.jetpass.dao.remote.api;

/*Generated by MPS */

import jetbrains.jetpass.api.IdItem;
import java.lang.reflect.Constructor;
import java.util.Set;
import java.util.Map;
import java.lang.reflect.Method;
import java.util.HashSet;
import java.lang.reflect.Proxy;
import java.lang.reflect.InvocationHandler;
import java.util.HashMap;
import java.lang.reflect.Type;
import java.lang.reflect.ParameterizedType;
import java.lang.reflect.WildcardType;

public class RemoteItemAccessor<I extends IdItem, J extends I> {
  private Class<I> itemInterface;
  private Class<J> dtoType;
  private RemoteItemStoreContainer storeContainer;
  private RemoteItemStore<I, J> dataStore;
  private Constructor<I> proxyConstructor;
  private Constructor<J> dtoConstructor;
  private Set<String> fieldWrapExclutions;
  private Map<Method, Integer> getter2fieldIndex;
  private Map<Method, String> getter2fieldName;
  private Map<Method, RemoteValueWrapper> getter2wrapper;
  private Map<String, Method> fieldName2getter;
  private Map<String, Method> fieldName2setter;

  public RemoteItemAccessor(Class<I> itemInterface, Class<J> dtoType, RemoteItemStoreContainer storeContainer, String... fieldWrapExclusions) {
    this.itemInterface = itemInterface;
    this.dtoType = dtoType;
    this.storeContainer = storeContainer;

    this.fieldWrapExclutions = new HashSet<String>();
    if (fieldWrapExclusions != null) {
      for (String fieldName : fieldWrapExclusions) {
        this.fieldWrapExclutions.add(fieldName);
      }
    }
  }

  public void init() {
    this.dataStore = (RemoteItemStore<I, J>) storeContainer.getRemoteDataStore(itemInterface);
    Class<?> proxyClass = Proxy.getProxyClass(itemInterface.getClassLoader(), new Class[]{itemInterface});
    try {
      this.proxyConstructor = (Constructor<I>) proxyClass.getConstructor(new Class[]{InvocationHandler.class});
      this.dtoConstructor = dtoType.getConstructor();
    } catch (NoSuchMethodException e) {
      throw new RuntimeException(e);
    }

    getter2fieldIndex = new HashMap<Method, Integer>();
    getter2fieldName = new HashMap<Method, String>();
    getter2wrapper = new HashMap<Method, RemoteValueWrapper>();
    fieldName2setter = new HashMap<String, Method>();
    fieldName2getter = new HashMap<String, Method>();

    int fieldIndex = 0;
    for (Method itemMethod : itemInterface.getMethods()) {
      String fieldName = getGetterFieldName(itemMethod);
      if (fieldName != null) {
        getter2fieldIndex.put(itemMethod, fieldIndex);
        getter2fieldName.put(itemMethod, fieldName);
        fieldName2setter.put(fieldName, findSetter(dtoType, fieldName, itemMethod.getReturnType()));
        fieldName2getter.put(fieldName, itemMethod);
        if (!(fieldWrapExclutions.contains(fieldName))) {
          createRemoteValueWrapper(itemMethod);
        }
        fieldIndex++;
      }
    }
  }

  public RemoteItemData<I, J> newRemoteItemData() {
    try {
      RemoteInvocationHandler<I, J> handler = new RemoteInvocationHandler<I, J>(this, dtoConstructor.newInstance());
      I item = this.proxyConstructor.newInstance(handler);
      return new RemoteItemData<I, J>(item, handler);
    } catch (Exception e) {
      throw new RuntimeException(e);
    }
  }

  public void setFieldValue(RemoteItemData<I, J> _this, String fieldName, J dto) {
    Method setter = fieldName2setter.get(fieldName);
    Method getter = fieldName2getter.get(fieldName);
    Integer fieldIndex = getter2fieldIndex.get(getter);
    if (fieldIndex == null || setter == null || getter == null) {
      throw new IllegalStateException("Unknown field " + fieldName);
    }
    try {
      Object value = getter.invoke(dto);
      _this.getHandler().setFieldValue(fieldIndex, setter, value);
    } catch (Exception e) {
      throw new RuntimeException(e);
    }
  }

  /*package*/ RemoteValueWrapper<?> getValueWrapper(Method getter) {
    return getter2wrapper.get(getter);
  }

  /*package*/ int getFieldGetterIndex(Method method) {
    Integer fieldIndex = getter2fieldIndex.get(method);
    if (fieldIndex == null) {
      // Method is not a getter 
      return -1;
    }
    return fieldIndex;
  }

  /*package*/ Integer getFieldIndex(String fieldName) {
    Method getter = fieldName2getter.get(fieldName);
    if (getter == null) {
      return null;
    }
    return getter2fieldIndex.get(getter);
  }

  /*package*/ void requestFieldValue(J dto, Method getter) {
    String id = dto.getId();
    String fieldName = getter2fieldName.get(getter);
    dataStore.requestFieldValue(id, fieldName);
  }

  private String getGetterFieldName(Method method) {
    if (method.getParameterTypes().length > 0) {
      return null;
    }
    if (void.class.equals(method.getReturnType())) {
      return null;
    }
    String methodName = method.getName();
    if ("getClass".equals(methodName)) {
      return null;
    }
    if (methodName.startsWith("is") && methodName.length() >= 3) {
      return Character.toLowerCase(methodName.charAt(2)) + methodName.substring(3);
    }
    if (methodName.startsWith("get") && methodName.length() >= 4) {
      return Character.toLowerCase(methodName.charAt(3)) + methodName.substring(4);
    }
    return null;
  }

  private Method findSetter(Class<?> type, String fieldName, Class fieldType) {
    String setterName = "set" + Character.toUpperCase(fieldName.charAt(0)) + fieldName.substring(1);
    for (Method method : type.getMethods()) {
      if (setterName.equals(method.getName()) && method.getParameterTypes().length == 1 && fieldType.isAssignableFrom(method.getParameterTypes()[0])) {
        return method;
      }
    }
    throw new IllegalStateException("Can't find setter method for field " + fieldName + " in class " + type.getName());
  }

  private void createRemoteValueWrapper(Method getter) {
    Class<?> returnType = getter.getReturnType();
    RemoteValueWrapper wrapper = getItemWrapper(returnType);
    if (Iterable.class.isAssignableFrom(returnType)) {
      // If getter returns Iterable<? extends IdItem> wrap returned value with RemoteIterableWrapper 
      Class elemetType = getElementType(getter.getGenericReturnType());
      RemoteValueWrapper valueWrapper = getItemWrapper(elemetType);
      if (valueWrapper != null) {
        wrapper = new RemoteIterableWrapper(valueWrapper);
      }
    }

    if (wrapper != null) {
      getter2wrapper.put(getter, wrapper);
    }
  }

  private RemoteValueWrapper getItemWrapper(Class<?> returnType) {
    if (returnType != null && IdItem.class.isAssignableFrom(returnType)) {
      // If getter returns IdItem wrap returned value with RemoteItemWrapper 
      RemoteItemStore store = storeContainer.getRemoteDataStore((Class<? extends IdItem>) returnType);
      if (store != null) {
        return new RemoteItemWrapper(store);
      } else {
        return new RemoteDynamicItemWrapper(storeContainer);
      }
    }
    return null;
  }

  private Class getElementType(Type iterableType) {
    // Supports classes in form Iterable<Resource> and Iterable<? extends Resource> 
    if (!(iterableType instanceof ParameterizedType)) {
      return null;
    }

    ParameterizedType parameterizedType = (ParameterizedType) iterableType;
    Type[] typeArgs = parameterizedType.getActualTypeArguments();
    if (typeArgs.length != 1) {
      return null;
    }

    Type elementType = typeArgs[0];
    if (elementType instanceof Class) {
      return (Class) elementType;
    }

    if (elementType instanceof WildcardType) {
      WildcardType wildcardType = (WildcardType) elementType;
      Type[] upperBounds = wildcardType.getUpperBounds();
      if (upperBounds.length != 1) {
        return null;
      }
      if (!(upperBounds[0] instanceof Class)) {
        return null;
      }
      return (Class) upperBounds[0];
    }

    return null;
  }

  public Class<I> getItemType() {
    return itemInterface;
  }

  public RemoteItemStore<I, J> getDataStore() {
    return dataStore;
  }
}
