package jetbrains.jetpass.dao.remote.api.event;

/*Generated by MPS */

import jetbrains.jetpass.api.event.Event;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import jetbrains.jetpass.api.event.Change;
import java.util.Iterator;
import java.util.Calendar;

public class MergedEvent implements Event {
  private static final int MERGED_EVENT_DEEP = 50;
  protected static Log log = LogFactory.getLog(MergedEvent.class);

  private final Event base;
  private final Event merged;
  private final int deep;

  public MergedEvent(Event base, Event merged) {
    int deep = 0;
    if (base instanceof MergedEvent) {
      MergedEvent mergedBase = ((MergedEvent) base);
      deep = mergedBase.deep + 1;
      if (deep > MERGED_EVENT_DEEP) {
        if (log.isWarnEnabled()) {
          log.warn("Merged event has deep more than " + MERGED_EVENT_DEEP + " please check events for target id: " + merged.getTargetId() + ". Current event: " + RemoteEventDAO.printEvent(merged));
        }
      }
    }
    this.deep = deep;
    this.base = base;
    this.merged = merged;
  }

  public String getTargetId() {
    return base.getTargetId();
  }

  public String getTargetType() {
    return base.getTargetType();
  }

  public String getTargetPresentation() {
    return base.getTargetPresentation();
  }

  public Iterable<? extends Change> getChanges() {
    return (Iterable<? extends Change>) new Iterable<Change>() {
      public Iterator<Change> iterator() {
        return new MergedEvent.JoinIterator<Change>((Iterable<Change>) base.getChanges(), (Iterable<Change>) merged.getChanges());
      }
    };
  }

  public String getType() {
    Event.Type baseType = Event.Type.fromString(base.getType());
    Event.Type mergedType = Event.Type.fromString(merged.getType());
    switch (baseType) {
      case UPDATE:
        // update always weaker 
        return mergedType.getName();
      case REMOVE:
        // remove is the strongest one 
        return baseType.getName();
      case CREATE:
        // remove is the strongest one 
        if (mergedType == Event.Type.REMOVE) {
          return mergedType.getName();
        }
      default:
        return baseType.getName();
    }
  }

  public Calendar getTimestamp() {
    Calendar baseDate = base.getTimestamp();
    Calendar mergedDate = merged.getTimestamp();
    return (mergedDate.before(baseDate) ?
      baseDate :
      mergedDate
    );
  }

  public String getAuthor() {
    return base.getAuthor();
  }

  public String getAuthorPresentation() {
    return base.getAuthorPresentation();
  }

  public String getAuthorType() {
    return base.getAuthorType();
  }

  public String getId() {
    return base.getId();
  }

  public Iterable<String> getAliasIds() {
    return base.getAliasIds();
  }

  private static class JoinIterator<T> implements Iterator<T> {
    private final Iterable<T> first;
    private final Iterable<T> second;
    private Iterator<T> firstIterator;
    private Iterator<T> secondIterator;

    private JoinIterator(Iterable<T> first, Iterable<T> second) {
      this.first = first;
      this.second = second;
    }

    public boolean hasNext() {
      return iter1().hasNext() || iter2().hasNext();
    }

    public T next() {
      return (iter1().hasNext() ?
        iter1().next() :
        iter2().next()
      );
    }

    public void remove() {
      throw new UnsupportedOperationException();
    }

    private Iterator<T> iter1() {
      if (firstIterator == null) {
        firstIterator = first.iterator();
      }
      return firstIterator;
    }

    private Iterator<T> iter2() {
      if (secondIterator == null) {
        secondIterator = second.iterator();
      }
      return secondIterator;
    }
  }
}
