package jetbrains.jetpass.dao.remote.api.event;

/*Generated by MPS */

import jetbrains.jetpass.dao.remote.api.RemoteDAO;
import jetbrains.jetpass.api.event.Event;
import jetbrains.jetpass.rest.dto.EventJSON;
import jetbrains.jetpass.client.accounts.Partial;
import jetbrains.jetpass.dao.api.event.EventDAO;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import java.text.DateFormat;
import java.text.SimpleDateFormat;
import java.util.concurrent.atomic.AtomicLong;
import jetbrains.jetpass.client.accounts.EventClient;
import java.util.TimeZone;
import jetbrains.mps.webr.rpc.rest.json.common.runtime.Page;
import jetbrains.jetpass.client.BaseFilter;
import jetbrains.jetpass.client.FieldPartial;
import jetbrains.jetpass.dao.api.DataAccessException;
import java.util.Iterator;
import java.util.Map;
import java.util.LinkedHashMap;

public class RemoteEventDAO extends RemoteDAO<Event, EventJSON, Partial.Event> implements EventDAO {
  protected static Log log = LogFactory.getLog(RemoteEventDAO.class);

  private final DateFormat dateFormat = new SimpleDateFormat("yyyy-MM-dd'T'HH:mm:ss.SSSZ");
  private final AtomicLong lastFetchedEventId = new AtomicLong(-1);
  private EventClient myClient;
  private int myPollingInterval;

  public RemoteEventDAO() {
    super(Partial.event(Partial.Event.ID, Partial.Event.TARGET_TYPE, Partial.Event.TARGET_ID, Partial.Event.TYPE));
    dateFormat.setTimeZone(TimeZone.getDefault());
  }

  protected Page<EventJSON> fetchPage(BaseFilter filter, FieldPartial<Partial.Event> partial) {
    return RemoteEventDAO.this.getClient().getEventPage(filter, partial);
  }

  protected EventJSON fetchItem(String id, FieldPartial<Partial.Event> partial) {
    return RemoteEventDAO.this.getClient().getEvent(id, partial);
  }

  protected FieldPartial<Partial.Event> createPartial(String fieldName) {
    return Partial.event(new Partial.Event(fieldName));
  }

  public void delete(String id) throws DataAccessException {
    throw new UnsupportedOperationException("Event deletion is prohibited!");
  }

  @Override
  protected void doSync(long syncStartTime) {
    this.syncDelta();
  }

  public Iterable<jetbrains.jetpass.api.event.Event> syncDelta() {
    long afterEventId = lastFetchedEventId.get();

    final FieldPartial<Partial.Event> partial = this._partial;
    String query = String.format("afterId: {%s}", String.valueOf(afterEventId));
    if (log.isInfoEnabled()) {
      log.info("Getting new events for query: " + query);
    }
    final BaseFilter filter = filter().query(query);
    Iterator<EventJSON> iter = createIterator(filter, partial);

    Map<String, jetbrains.jetpass.api.event.Event> newEvents = new LinkedHashMap<String, jetbrains.jetpass.api.event.Event>(1);
    for (EventJSON dto; iter.hasNext();) {
      dto = iter.next();

      onNewItem(dto);

      jetbrains.jetpass.api.event.Event event = putRemoteItem(store, dto, partial).getItem();
      String id = event.getTargetId();
      jetbrains.jetpass.api.event.Event existent = newEvents.get(id);
      if (log.isDebugEnabled()) {
        log.debug("New Event: " + RemoteEventDAO.printEvent(event));
      }
      if (existent != null) {
        // merge events 
        event = new MergedEvent(existent, event);
      }
      newEvents.put(id, event);
    }
    if (log.isInfoEnabled()) {
      log.info("New Events recieved: " + newEvents.size());
    }
    if (log.isInfoEnabled()) {
      log.info("New merged Events: \n\t" + dumpEvents(newEvents.values()));
    }

    return newEvents.values();
  }

  public void init() {
    jetbrains.jetpass.api.event.Event first = getAllItems().orderBy("id", false).first();
    if (first != null && first.getId() != null) {
      if (log.isDebugEnabled()) {
        log.debug("First seen event id was: " + first.getId());
      }
      lastFetchedEventId.set(Long.parseLong(first.getId()));
    } else {
      if (log.isDebugEnabled()) {
        log.debug("No events yet");
      }
    }
  }

  @Override
  protected void onNewItem(final EventJSON dto) {
    // BC! Assume that all eventId are longs 
    long newId = Long.parseLong(dto.getId());
    long oldId = lastFetchedEventId.get();
    if (newId > oldId) {
      lastFetchedEventId.set(newId);
    }
  }

  public EventClient getClient() {
    return this.myClient;
  }

  public void setClient(EventClient value) {
    this.myClient = value;
  }

  public int getPollingInterval() {
    return this.myPollingInterval;
  }

  public void setPollingInterval(int value) {
    this.myPollingInterval = value;
  }

  private static String dumpEvents(Iterable<jetbrains.jetpass.api.event.Event> events) {
    StringBuilder builder = new StringBuilder();
    boolean first = true;
    for (jetbrains.jetpass.api.event.Event evt : events) {
      if (!(first)) {
        builder.append("\n\t");
      }
      first = false;
      builder.append(printEvent(evt));
    }
    return builder.toString();
  }

  protected static String printEvent(jetbrains.jetpass.api.event.Event event) {
    return event.getType() + ":" + event.getTargetType() + ":" + event.getTargetId();
  }
}
