package jetbrains.jetpass.dao.remote.api.security;

/*Generated by MPS */

import jetbrains.jetpass.dao.remote.api.RemoteNamedMutableDAO;
import jetbrains.jetpass.api.security.Project;
import jetbrains.jetpass.rest.dto.ProjectJSON;
import jetbrains.jetpass.client.accounts.Partial;
import jetbrains.jetpass.dao.api.security.ProjectDAO;
import jetbrains.jetpass.client.accounts.ProjectClient;
import jetbrains.mps.webr.rpc.rest.json.common.runtime.Page;
import jetbrains.jetpass.client.BaseFilter;
import jetbrains.jetpass.client.FieldPartial;
import jetbrains.jetpass.sequence.Sequence;
import jetbrains.jetpass.sequence.impl.FilterFunction;
import org.jetbrains.annotations.Nullable;
import jetbrains.jetpass.dao.api.DataAccessException;
import java.util.List;
import jetbrains.jetpass.sequence.impl.IterableSequence;
import jetbrains.jetpass.sequence.impl.TransformFunction;
import javax.ws.rs.WebApplicationException;
import jetbrains.jetpass.api.security.Team;
import jetbrains.jetpass.rest.dto.TeamJSON;
import jetbrains.jetpass.sequence.impl.ListSequence;
import jetbrains.jetpass.sequence.impl.EmptySequence;
import jetbrains.jetpass.rest.dto.UserJSON;
import jetbrains.jetpass.rest.dto.UserGroupJSON;

public class RemoteProjectDAO extends RemoteNamedMutableDAO<Project, ProjectJSON, Partial.Project> implements ProjectDAO {
  private volatile String globalProjectId = null;
  private ProjectClient myClient;

  public RemoteProjectDAO() {
    super(Partial.project(Partial.Project.ID, Partial.Project.ALIAS_IDS, Partial.Project.NAME));
  }

  protected Page<ProjectJSON> fetchPage(BaseFilter filter, FieldPartial<Partial.Project> partial) {
    return RemoteProjectDAO.this.getClient().getProjectPage(filter, partial);
  }

  @Override
  public Sequence<jetbrains.jetpass.api.security.Project> getAllItems() {
    return getAllItems(false);
  }

  public Sequence<jetbrains.jetpass.api.security.Project> getAllItems(boolean withArchived) {
    Sequence<jetbrains.jetpass.api.security.Project> all = super.getAllItems();
    return (withArchived ?
      all :
      all.where(new FilterFunction<jetbrains.jetpass.api.security.Project>() {
        public boolean matches(jetbrains.jetpass.api.security.Project it) {
          return !((it.isArchived()));
        }
      })
    );
  }

  @Override
  protected BaseFilter filter() {
    return ProjectClient.filter().archived(true);
  }

  protected ProjectJSON fetchItem(String id, FieldPartial<Partial.Project> partial) {
    return RemoteProjectDAO.this.getClient().getProject(id, partial);
  }

  protected FieldPartial<Partial.Project> createPartial(String fieldName) {
    return Partial.project(new Partial.Project(fieldName));
  }

  protected ProjectJSON doCreate(ProjectJSON json, FieldPartial<Partial.Project> partial) {
    return RemoteProjectDAO.this.getClient().createProject(json, partial);
  }

  protected void doUpdate(String id, ProjectJSON change) {
    RemoteProjectDAO.this.getClient().updateProject(id, change);
  }

  protected void doDelete(String id, @Nullable String successor) {
    RemoteProjectDAO.this.getClient().deleteProject(id, successor);
  }

  public void removeResource(String projectId, String resourceId) throws DataAccessException {
    getNotNull(projectId);

    RemoteProjectDAO.this.getRemoteAPI().getResourceDAO().getNotNull(resourceId);

    RemoteProjectDAO.this.getClient().getResourcesClient(projectId).removeResource(resourceId);
    RemoteProjectDAO.this.getRemoteAPI().syncNow();
  }

  public void addResource(String projectId, String resourceId) throws DataAccessException {
    getNotNull(projectId);

    RemoteResourceDAO resourceDAO = this.getRemoteAPI().getResourceDAO();
    RemoteProjectDAO.this.getClient().getResourcesClient(projectId).addResource(resourceDAO.toJSON(resourceId), resourceDAO.getDefaultPartial());
    this.getRemoteAPI().syncNow();
  }

  @Override
  protected void doSync(long syncStartTime) {
    this.globalProjectId = null;
    super.doSync(syncStartTime);
  }

  public jetbrains.jetpass.api.security.Project getGlobal() {
    if (this.globalProjectId == null) {
      this.globalProjectId = RemoteProjectDAO.this.getClient().global(_partial).getId();
    }
    return get(globalProjectId);
  }

  public jetbrains.jetpass.api.security.Project merge(Iterable<jetbrains.jetpass.api.security.Project> projects, String key, String name, String description, Boolean archived) throws DataAccessException {
    try {
      List<ProjectJSON> list = new IterableSequence(projects).select(new TransformFunction<jetbrains.jetpass.api.security.Project, ProjectJSON>() {
        public ProjectJSON transform(jetbrains.jetpass.api.security.Project it) {
          return toJSON(it);
        }
      }).toListSequence().toList();
      String mergedId = RemoteProjectDAO.this.getClient().merge(key, name, description, archived, list.<ProjectJSON>toArray(new ProjectJSON[list.size()]), Partial.project(Partial.Project.ID)).getId();
      this.getRemoteAPI().syncNow();
      return get(mergedId);
    } catch (WebApplicationException ex) {
      throw wrapClientException(ex, null);
    }
  }

  public Sequence<Team> getTeams(String projectId) throws DataAccessException {
    try {
      ProjectClient.TeamClient teamsClient = this.getClient().getTeamsClient(projectId);
      List<TeamJSON> teams = teamsClient.getTeamPage(ProjectClient.TeamClient.filter().topAll(), getTeamFeilds()).getItems();
      if (teams != null) {
        return ListSequence.fromIterable(teams).select(new TransformFunction<TeamJSON, Team>() {
          public Team transform(TeamJSON team) {
            return fromJSON(team);
          }
        });
      } else {
        return EmptySequence.getInstance();
      }
    } catch (WebApplicationException e) {
      throw wrapClientException(e, null);
    }
  }

  public Team getWholeTeam(String projectId) throws DataAccessException {
    try {
      ProjectClient.TeamClient teamsClient = this.getClient().getTeamsClient(projectId);
      TeamJSON wholeTeam = teamsClient.whole(getTeamFeilds());
      return fromJSON(wholeTeam);
    } catch (WebApplicationException e) {
      throw wrapClientException(e, null);
    }
  }

  public Team getTeam(String projectId, String roleId) throws DataAccessException {
    try {
      ProjectClient.TeamClient teamsClient = this.getClient().getTeamsClient(projectId);
      return fromJSON(teamsClient.getTeam(roleId, getTeamFeilds()));
    } catch (WebApplicationException e) {
      throw wrapClientException(e, null);
    }
  }

  public Team addTeam(String projectId, Team team) throws DataAccessException {
    try {
      ProjectClient.TeamClient teamsClient = this.getClient().getTeamsClient(projectId);
      Team result = fromJSON(teamsClient.addTeam(new TeamJSON(team), getTeamFeilds()));
      this.getRemoteAPI().syncNow();
      return result;
    } catch (WebApplicationException e) {
      throw wrapClientException(e, null);
    }
  }

  private Partial<Partial.Team> getTeamFeilds() {
    return Partial.team(Partial.Team.ROLE(Partial.Role.ID), Partial.Team.USERS(Partial.User.ID), Partial.Team.GROUPS(Partial.UserGroup.ID));
  }

  private jetbrains.jetpass.api.security.Team fromJSON(final TeamJSON team) {
    return new jetbrains.jetpass.api.security.Team() {
      public jetbrains.jetpass.api.security.Role getRole() {
        return RemoteProjectDAO.this.getRemoteAPI().getRoleDAO().get(team.getRole().getId());
      }

      public Iterable<jetbrains.jetpass.api.authority.User> getUsers() {
        return ListSequence.fromIterable(team.getUsers()).select(new TransformFunction<UserJSON, jetbrains.jetpass.api.authority.User>() {
          public jetbrains.jetpass.api.authority.User transform(UserJSON json) {
            return RemoteProjectDAO.this.getRemoteAPI().getUserDAO().get(json.getId());
          }
        });
      }

      public Iterable<jetbrains.jetpass.api.authority.UserGroup> getGroups() {
        return ListSequence.fromIterable(team.getGroups()).select(new TransformFunction<UserGroupJSON, jetbrains.jetpass.api.authority.UserGroup>() {
          public jetbrains.jetpass.api.authority.UserGroup transform(UserGroupJSON json) {
            return RemoteProjectDAO.this.getRemoteAPI().getUserGroupDAO().get(json.getId());
          }
        });
      }
    };
  }

  protected ProjectJSON toJSON(jetbrains.jetpass.api.security.Project item) {
    return (item instanceof ProjectJSON ?
      (ProjectJSON) item :
      new ProjectJSON(item)
    );
  }

  public ProjectClient getClient() {
    return this.myClient;
  }

  public void setClient(ProjectClient value) {
    this.myClient = value;
  }
}
