package jetbrains.jetpass.dao.remote.api.security;

/*Generated by MPS */

import jetbrains.jetpass.dao.remote.api.RemoteNamedMutableDAO;
import jetbrains.jetpass.api.security.Role;
import jetbrains.jetpass.rest.dto.RoleJSON;
import jetbrains.jetpass.client.accounts.Partial;
import jetbrains.jetpass.dao.api.security.RoleDAO;
import jetbrains.jetpass.client.accounts.RoleClient;
import jetbrains.mps.webr.rpc.rest.json.common.runtime.Page;
import jetbrains.jetpass.client.BaseFilter;
import jetbrains.jetpass.client.FieldPartial;
import org.jetbrains.annotations.Nullable;
import jetbrains.jetpass.dao.api.DataAccessException;
import javax.ws.rs.WebApplicationException;
import jetbrains.jetpass.rest.dto.ServiceJSON;
import java.util.List;
import jetbrains.jetpass.sequence.impl.IterableSequence;
import jetbrains.jetpass.sequence.impl.TransformFunction;
import jetbrains.jetpass.dao.api.ItemNotFoundException;

public class RemoteRoleDAO extends RemoteNamedMutableDAO<Role, RoleJSON, Partial.Role> implements RoleDAO {
  private RoleClient myClient;

  public RemoteRoleDAO() {
    super(Partial.role(Partial.Role.ID, Partial.Role.ALIAS_IDS, Partial.Role.PERMISSIONS(Partial.Permission.ID)));
  }

  protected Page<RoleJSON> fetchPage(BaseFilter filter, FieldPartial<Partial.Role> partial) {
    return RemoteRoleDAO.this.getClient().getRolePage(filter, partial);
  }

  protected RoleJSON fetchItem(String id, FieldPartial<Partial.Role> partial) {
    return RemoteRoleDAO.this.getClient().getRole(id, partial);
  }

  protected FieldPartial<Partial.Role> createPartial(String fieldName) {
    return Partial.role(new Partial.Role(fieldName));
  }

  protected RoleJSON doCreate(RoleJSON json, FieldPartial<Partial.Role> partial) {
    return RemoteRoleDAO.this.getClient().createRole(json, partial);
  }

  protected void doUpdate(String id, RoleJSON change) {
    RemoteRoleDAO.this.getClient().updateRole(id, change);
  }

  protected void doDelete(String id, @Nullable String successor) {
    RemoteRoleDAO.this.getClient().deleteRole(id, successor);
  }

  public void addPermission(String roleId, String permissionId) throws DataAccessException {
    getNotNull(roleId);

    RemotePermissionDAO permissionDAO = this.getRemoteAPI().getPermissionDAO();
    try {
      RemoteRoleDAO.this.getClient().getPermissionsClient(roleId).addPermission(permissionDAO.toJSON(permissionId), permissionDAO.getDefaultPartial());
    } catch (WebApplicationException ex) {
      throw wrapClientException(ex, roleId);
    }
    this.getRemoteAPI().syncNow();
  }

  public void removePermission(String roleId, String permissionId) throws DataAccessException {
    getNotNull(roleId);

    RemotePermissionDAO permissionDAO = this.getRemoteAPI().getPermissionDAO();
    permissionDAO.getNotNull(permissionId);
    try {
      RemoteRoleDAO.this.getClient().getPermissionsClient(roleId).removePermission(permissionId);
    } catch (WebApplicationException ex) {
      throw wrapClientException(ex, roleId);
    }

    this.getRemoteAPI().syncNow();
  }

  public void resetPermissions(String roleId, String serviceId) throws DataAccessException {
    getNotNull(roleId);
    ServiceJSON service = null;
    if (serviceId != null) {
      service = toServiceJSON(serviceId);
    }
    try {
      RemoteRoleDAO.this.getClient().reset(roleId, service, this.getDefaultPartial());
    } catch (WebApplicationException ex) {
      throw wrapClientException(ex, roleId);
    }
    this.getRemoteAPI().syncNow();
  }

  public void addServicePermissions(String roleId, String serviceId) throws DataAccessException {
    getNotNull(roleId);
    ServiceJSON service = toServiceJSON(serviceId);
    try {
      RemoteRoleDAO.this.getClient().servicepermissions(roleId, true, null, service, this.getDefaultPartial());
    } catch (WebApplicationException ex) {
      throw wrapClientException(ex, roleId);
    }
    this.getRemoteAPI().syncNow();
  }

  public void removeServicePermissions(String roleId, String serviceId) throws DataAccessException {
    getNotNull(roleId);
    ServiceJSON service = toServiceJSON(serviceId);
    try {
      RemoteRoleDAO.this.getClient().servicepermissions(roleId, null, true, service, this.getDefaultPartial());
    } catch (WebApplicationException ex) {
      throw wrapClientException(ex, roleId);
    }
    this.getRemoteAPI().syncNow();
  }

  public jetbrains.jetpass.api.security.Role merge(Iterable<jetbrains.jetpass.api.security.Role> roles, String key, String name, String description) throws DataAccessException {
    try {
      List<RoleJSON> list = new IterableSequence(roles).select(new TransformFunction<jetbrains.jetpass.api.security.Role, RoleJSON>() {
        public RoleJSON transform(jetbrains.jetpass.api.security.Role it) {
          return toJSON(it);
        }
      }).toListSequence().toList();
      String mergedId = RemoteRoleDAO.this.getClient().merge(key, name, description, list.<RoleJSON>toArray(new RoleJSON[list.size()]), Partial.role(Partial.Role.ID)).getId();
      this.getRemoteAPI().syncNow();
      return get(mergedId);
    } catch (WebApplicationException e) {
      throw wrapClientException(e, null);
    }
  }

  public RoleJSON toJSON(jetbrains.jetpass.api.security.Role item) {
    return (item instanceof RoleJSON ?
      (RoleJSON) item :
      new RoleJSON(item)
    );
  }

  protected RoleJSON toJSON(String roleId) {
    RoleJSON role = new RoleJSON();
    role.setId(roleId);
    return role;
  }

  protected ServiceJSON toServiceJSON(String serviceId) throws ItemNotFoundException {
    this.getRemoteAPI().getServiceDAO().getNotNull(serviceId);
    ServiceJSON serviceJSON = new ServiceJSON();
    serviceJSON.setId(serviceId);
    return serviceJSON;
  }

  public RoleClient getClient() {
    return this.myClient;
  }

  public void setClient(RoleClient value) {
    this.myClient = value;
  }
}
