package jetbrains.jetpass.dao.util;

public class PersistentHashMap<K, V> {

    private final PersistentHashSet<Entry<K, V>> set;

    public PersistentHashMap() {
        set = new PersistentHashSet<Entry<K, V>>();
    }

    private PersistentHashMap(final AbstractPersistentHashSet.RootTableNode<Entry<K, V>> root) {
        set = new PersistentHashSet<Entry<K, V>>(root);
    }

    public ImmutablePersistentHashMap getCurrent() {
        return new ImmutablePersistentHashMap();
    }

    public PersistentHashMap<K, V> getClone() {
        return new PersistentHashMap<K, V>(set.getRoot());
    }

    public MutablePersistentHashMap beginWrite() {
        return new MutablePersistentHashMap();
    }

    public boolean endWrite(final MutablePersistentHashMap tree) {
        return set.endWrite(tree);
    }

    public class ImmutablePersistentHashMap extends PersistentHashSet.ImmutablePersistentHashSet<Entry<K, V>> {

        ImmutablePersistentHashMap() {
            super(set.getRoot());
        }

        public V get(final K key) {
            final Entry<K, V> entry = getRoot().getKey(new Entry<K, V>(key), key.hashCode(), 0);
            return entry == null ? null : entry.getValue();
        }

        public boolean containsKey(final K key) {
            return getRoot().getKey(new Entry<K, V>(key), key.hashCode(), 0) != null;
        }
    }

    public class MutablePersistentHashMap extends PersistentHashSet.MutablePersistentHashSet<Entry<K, V>> {

        MutablePersistentHashMap() {
            super(set);
        }

        public V get(final K key) {
            final Entry<K, V> entry = getRoot().getKey(new Entry<K, V>(key), key.hashCode(), 0);
            return entry == null ? null : entry.getValue();
        }

        public boolean containsKey(final K key) {
            return getRoot().getKey(new Entry<K, V>(key), key.hashCode(), 0) != null;
        }

        public void put(final K key, final V value) {
            add(new Entry<K, V>(key, value));
        }

        @SuppressWarnings("MethodOverloadsMethodOfSuperclass")
        public V removeKey(final K key) {
            final Entry<K, V> entry = getRoot().getKey(new Entry<K, V>(key), key.hashCode(), 0);
            final V result = entry == null ? null : entry.getValue();
            if (entry != null) {
                remove(entry);
            }
            return result;
        }
    }

    public static class Entry<K, V> {


        private final K key;
        private final V value;

        private Entry(K key) {
            this(key, null);
        }

        private Entry(final K key, final V value) {
            this.key = key;
            this.value = value;
        }

        @SuppressWarnings("EqualsWhichDoesntCheckParameterClass")
        @Override
        public boolean equals(Object obj) {
            //noinspection unchecked
            return key.equals(((Entry<K, V>) obj).key);

        }

        @Override
        public int hashCode() {
            return key.hashCode();
        }


        public K getKey() {
            return key;
        }

        public V getValue() {
            return value;
        }
    }
}