package jetbrains.jetpass.pojo.api.authority.profile;

/*Generated by MPS */

import java.util.regex.Pattern;
import org.jetbrains.annotations.Nullable;
import org.jetbrains.annotations.NotNull;
import java.util.regex.Matcher;
import java.net.URI;
import java.net.URISyntaxException;

public class LoginUtils {
  private static final char[] SUBSTITUTION_TABLE;
  private static final char DEFAULT_SUBSTITUTION_CHAR = '_';
  private static final int ASCII_TABLE_SIZE = 128;
  private static final int LOGIN_MIN_LENGTH = 2;
  private static final int LOGIN_MAX_LENGTH = 32;
  private static final char DOT = '.';
  private static final String EMPTY_LOGIN_ERROR = "Empty";
  private static final Pattern INCREMENTED_LOGIN = Pattern.compile("(.+)\\.\\d+");

  @Nullable
  public static String getLoginFromEmail(String email) {
    if ((email == null || email.length() == 0)) {
      return null;
    }
    int indexOfAt;
    return ((indexOfAt = email.indexOf("@")) > 0 ?
      email.substring(0, indexOfAt) :
      null
    );
  }

  @NotNull
  public static String getBestMatchingLogin(String username, String firstName, String lastName) {
    if ((username == null || username.length() == 0)) {
      return null;
    }
    int indexOfAt;
    return ((indexOfAt = username.indexOf("@")) > 0 ?
      username.substring(0, indexOfAt) :
      null
    );
  }

  @Nullable
  public static String loginFromFirstLastName(@Nullable String firstName, @Nullable String lastName) {
    if ((firstName == null || firstName.length() == 0)) {
      return lastName;
    }
    if ((lastName == null || lastName.length() == 0)) {
      return firstName;
    }

    return firstName + "." + lastName;
  }

  public static boolean isValidLogin(@Nullable final String login) {
    if (login == null) {
      return false;
    }

    final int sz = login.length();
    if (sz < LOGIN_MIN_LENGTH || sz > LOGIN_MAX_LENGTH) {
      return false;
    }
    for (int i = 0; i < sz; i++) {
      char chr = login.charAt(i);
      if (chr >= ASCII_TABLE_SIZE) {
        if (!(Character.isLetterOrDigit(chr))) {
          return false;
        }
      } else if (SUBSTITUTION_TABLE[chr] != chr) {
        return false;
      }
    }
    return true;
  }

  @Nullable
  public static String getErrorReason(@Nullable final String login) {
    if ((login == null || login.length() == 0)) {
      return EMPTY_LOGIN_ERROR;
    }

    final int sz = login.length();
    if (sz < LOGIN_MIN_LENGTH || sz > LOGIN_MAX_LENGTH) {
      return (sz < LOGIN_MIN_LENGTH ?
        "Too short" :
        "Too long"
      );
    }

    for (int i = 0; i < sz; i++) {
      char chr = login.charAt(i);
      if (chr >= ASCII_TABLE_SIZE) {
        if (!(Character.isLetterOrDigit(chr))) {
          return "Is not unicode letter or digit [" + chr + "]";
        }
      } else if (SUBSTITUTION_TABLE[chr] != chr) {
        return "Deprecated character [" + chr + "]";
      }
    }

    return null;
  }

  @NotNull
  public static String generate() {
    return generate("");
  }

  @NotNull
  public static String generate(@NotNull String prefix) {
    // Use simple 
    return fix(new StringBuilder(prefix).append(Long.toHexString(Double.doubleToLongBits(Math.random()))).toString());
  }

  @Nullable
  public static String fix(@Nullable final String login) {
    if ((login == null || login.length() == 0)) {
      return null;
    }

    final int originalSize = login.length();
    final int resultSize;
    if (originalSize < LOGIN_MIN_LENGTH) {
      resultSize = LOGIN_MIN_LENGTH;
    } else if (originalSize > LOGIN_MAX_LENGTH) {
      resultSize = LOGIN_MAX_LENGTH;
    } else {
      resultSize = originalSize;
    }

    char[] buffer = new char[resultSize];
    for (int i = 0; i < resultSize; i++) {
      char chr;
      if (i >= originalSize) {
        chr = DEFAULT_SUBSTITUTION_CHAR;
      } else {
        chr = login.charAt(i);
        if (chr >= ASCII_TABLE_SIZE) {
          chr = (Character.isLetterOrDigit(chr) ?
            chr :
            DEFAULT_SUBSTITUTION_CHAR
          );
        } else {
          chr = SUBSTITUTION_TABLE[chr];
        }
      }
      buffer[i] = chr;
    }
    return new String(buffer);
  }

  public static String cutLoginCounter(@NotNull String login) {
    Matcher matcher = INCREMENTED_LOGIN.matcher(login);
    if (matcher.matches()) {
      login = matcher.group(1);
    }
    return login;
  }

  @NotNull
  public static String increment(@NotNull String login, final int inc) {
    if (inc < 0) {
      throw new IllegalArgumentException("Can't increment on negative value");
    }
    if ((login == null || login.length() == 0)) {
      throw new IllegalArgumentException("Can't increment empty string");
    }

    login = cutLoginCounter(login);

    int sz = login.length();
    // result size is "login" + "." + inc 
    String suffix = Integer.toString(inc);
    final int ext = suffix.length() + 1;
    int resSz = sz + ext;
    if (resSz > LOGIN_MAX_LENGTH) {
      resSz = LOGIN_MAX_LENGTH;
    } else if (resSz < LOGIN_MIN_LENGTH) {
      resSz = LOGIN_MIN_LENGTH;
    }

    final int strLength = resSz - ext;
    final char[] buffer = new char[strLength];
    for (int i = 0; i < strLength; i++) {
      char chr = DEFAULT_SUBSTITUTION_CHAR;
      if (i < sz) {
        chr = login.charAt(i);
      }
      buffer[i] = chr;
    }

    return new StringBuilder(resSz).append(buffer).append(DOT).append(suffix).toString();
  }

  public static String tryStripFromUri(String uri) {
    if ((uri == null || uri.length() == 0)) {
      return uri;
    }
    URI u;
    try {
      int len = uri.length();
      StringBuilder res = new StringBuilder(len);
      u = new URI((uri.endsWith("/") ?
        uri.substring(0, len - 1) :
        uri
      ));
      boolean empty = false;
      if (isNotEmpty_1yfm2c_a0e0c0k(u.getUserInfo())) {
        res.append(u.getUserInfo());
      } else if (isNotEmpty_1yfm2c_a0a4a2a01(u.getHost())) {
        res.append(u.getHost());
      } else {
        empty = true;
      }
      if (isNotEmpty_1yfm2c_a0f0c0k(u.getPath())) {
        empty = false;
        res.append(u.getPath());
      }
      if (!(empty)) {
        uri = res.toString();
      }
    } catch (URISyntaxException ex) {
      // ignore 
    }
    return uri;
  }

  public static boolean isNotEmpty_1yfm2c_a0e0c0k(String str) {
    return str != null && str.length() > 0;
  }

  public static boolean isNotEmpty_1yfm2c_a0a4a2a01(String str) {
    return str != null && str.length() > 0;
  }

  public static boolean isNotEmpty_1yfm2c_a0f0c0k(String str) {
    return str != null && str.length() > 0;
  }

  static {
    SUBSTITUTION_TABLE = new char[ASCII_TABLE_SIZE];
    for (int i = 0; i < SUBSTITUTION_TABLE.length; i++) {
      SUBSTITUTION_TABLE[i] = ((char) (Character.isLetterOrDigit(i) ?
        i :
        DEFAULT_SUBSTITUTION_CHAR
      ));
    }
    SUBSTITUTION_TABLE['$'] = '$';
    SUBSTITUTION_TABLE['+'] = '+';
    SUBSTITUTION_TABLE['.'] = '.';
    SUBSTITUTION_TABLE['-'] = '-';
    SUBSTITUTION_TABLE['='] = '=';
    SUBSTITUTION_TABLE['|'] = '|';
    SUBSTITUTION_TABLE['@'] = '@';
    SUBSTITUTION_TABLE['\\'] = DOT;
    SUBSTITUTION_TABLE['/'] = DOT;
    SUBSTITUTION_TABLE['{'] = DOT;
    SUBSTITUTION_TABLE['}'] = DOT;
    SUBSTITUTION_TABLE['('] = DOT;
    SUBSTITUTION_TABLE[')'] = DOT;
  }
}
