package jetbrains.jetpass.sequence.impl;

/*Generated by MPS */

import jetbrains.jetpass.sequence.BaseSequence;
import jetbrains.jetpass.sequence.Sequence;
import java.util.Iterator;
import java.util.NoSuchElementException;

public class SkipSequence<T> extends BaseSequence<T> {
  private final Sequence<T> input;
  private final int count;

  public SkipSequence(final Sequence<T> input, final int count) {
    this.input = input;
    this.count = count;
  }

  public Iterator<T> iterator() {
    Iterator<T> source = input.iterator();
    if (count < 0) {
      return source;
    } else {
      return new SkipSequence.SkipIterator(source, count);
    }
  }

  public int size() {
    int inputSize = input.size();
    return Math.max(inputSize - count, 0);
  }

  @Override
  public boolean isQueriedSingleton() {
    return count == 0 && input.isQueriedSingleton();
  }

  private static class SkipIterator<T> implements Iterator<T> {
    private Iterator<T> source = null;
    private int skipped = 0;
    private int count;

    public SkipIterator(Iterator<T> source, int count) {
      this.source = source;
      this.count = count;
    }

    public boolean hasNext() {
      if (skipped <= count) {
        this.move();
      }
      return source.hasNext();
    }

    public void move() {
      while (skipped < count && source.hasNext()) {
        source.next();
        skipped++;
      }
      if (!(source.hasNext())) {
        skipped = count;
      }
    }

    public T next() {
      if (!(hasNext())) {
        throw new NoSuchElementException();
      }
      return source.next();
    }

    public void remove() {
      source.remove();
    }
  }
}
