package jetbrains.jetpass.sequence.impl;

/*Generated by MPS */

import jetbrains.jetpass.sequence.BaseSequence;
import jetbrains.jetpass.sequence.Sequence;
import java.util.Iterator;
import java.util.NoSuchElementException;

public class TakeSequence<T> extends BaseSequence<T> {
  private final Sequence<T> input;
  private final int count;

  public TakeSequence(final Sequence<T> input, final int count) {
    this.input = input;
    this.count = count;
  }

  public Iterator<T> iterator() {
    Iterator<T> source = input.iterator();
    if (count < 0) {
      return source;
    } else {
      return new TakeSequence.TakeIterator(source, count);
    }
  }

  public int size() {
    int inputSize = input.size();
    return Math.min(inputSize, count);
  }

  @Override
  public boolean isQueriedSingleton() {
    return input.isQueriedSingleton();
  }

  private static class TakeIterator<T> implements Iterator<T> {
    private Iterator<T> source = null;
    private int taken = 0;
    private int count;

    public TakeIterator(Iterator<T> source, int count) {
      this.source = source;
      this.count = count;
    }

    public boolean hasNext() {
      if (taken >= count) {
        return false;
      }
      return source.hasNext();
    }

    public T next() {
      if (!(hasNext())) {
        throw new NoSuchElementException();
      }
      T next = source.next();
      taken++;
      return next;
    }

    public void remove() {
      source.remove();
    }
  }
}
